
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/


#include "Stdafx.h"
#include "Khan.h"
#include "KhanDB.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////
// CVideo
CVideo::CVideo()
{
}
CVideo::~CVideo()
{
}
CVideo::CVideo(const CVideo & video) 
{ 
	Copy(video); 
}
void CVideo::operator=(const CVideo & video) 
{ 
	Copy(video); 
};
BOOL CVideo::operator==(const CVideo & video)
{
	if(	topic		== video.topic		 &&
		date_added	== video.date_added  &&
		description == video.description &&
		mp4			== video.mp4		 &&
		duration	== video.duration	 &&
		title		== video.title)
		return TRUE;
	return FALSE;
}
BOOL CVideo::operator!=(const CVideo & video) 
{ 
	return !(*this == video); 
}
void CVideo::Copy(const CVideo & video)
{
	topic		= video.topic;
	date_added	= video.date_added;
	description = video.description;
	mp4			= video.mp4;
	duration	= video.duration;
	title		= video.title; 
}	
void CVideo::Zero() 
{
	topic.Empty();
	date_added.Empty();
	description.Empty();
	mp4.Empty();
	duration.Empty();
	title.Empty(); 
}

// Not used - delete
//
// Get a file name for a video - based on its Title.
//
// Note: This was the original way to name video files, but 
// was a BAD mistake as Khan can change the title.
//
// We now use GetShortFileName() instead.
//
BOOL CVideo::GetShortFileNameTitleOld(CString & name) // "short" compared to the mp4 url
{
	// Clean up title.
	TCHAR c;
	TCHAR dash = _T('-');
	CString temp = title;
	temp.MakeLower();
	int nChars = temp.GetLength();
	for(int i = 0; i < nChars; i++)
	{
		c = temp.GetAt(i);
		if(c < 48)
			temp.SetAt(i, dash);
		else if(c > 57 && c < 97)
			temp.SetAt(i, dash);
		else if(c > 123)
			temp.SetAt(i, dash);
	}

	// Get type.
#if VS2005
	int i = mp4.ReverseFind(_T('.'));	//vs2005
#else
	i = mp4.ReverseFind(_T('.'));		// vc6
#endif
	if(i == -1)
		return FALSE;

	// Append type to title.
	name = temp;
	name += mp4.Right(mp4.GetLength() - i);

	return TRUE;
}

// Get file name for a video from the mp4 member string.
BOOL CVideo::GetShortFileName(CString & name)
{
	// For example, get:
	//	frBJEYvyd-8.mp4  
	// from the mp4:
	//	http://s3.amazonaws.com/KA-youtube-converted/frBJEYvyd-8.mp4/frBJEYvyd-8.mp4
	//
	// Get chars right of first / from the end.
	int i = mp4.ReverseFind(_T('/'));
	if(i != -1)
	{
		int len = mp4.GetLength();
		name = mp4.Right(len - (i + 1));
		return TRUE;
	}
	// No go
	return FALSE;
}

// Compare a file name created with GetShortFileNameTitle()
// to the fileNameTitle param.
BOOL CVideo::CompareFileNamesTitleOld(const CString & fileNameTitle)
{
	// We have to build a file name as does GetShortFileNameTitle()
	// to compare it. So to speed up we check chars as we build.
	//
	// Note GetShortFileNameTitle() does not change the length of a 
	// file name, except adding the extension.
	//
	// First, check lengths. Note fileNameTitle has 4 extra chars (.mp4).
	int nChars1 = title.GetLength();
	int nChars2 = fileNameTitle.GetLength();
	if(nChars1 != nChars2 - 4)
		return FALSE;

	TCHAR c1, c2;
	TCHAR dash = _T('-');

	// Get lower case title.
	CString temp = title;
	temp.MakeLower();

	// Loop chars less the .mp4 .
	for(int i = 0; i < nChars1; i++)
	{
		c1 = temp.GetAt(i);
		c2 = fileNameTitle.GetAt(i);

		// Check chars that would get converted to dashes.
		// Same as GetShortFileName().
		if(c1 < 48)
		{
			if(c2 != dash)
				return FALSE;
		}
		else if(c1 > 57 && c1 < 97)
		{
			if(c2 != dash)
				return FALSE;
		}
		else if(c1 > 123)
		{
			if(c2 != dash)
				return FALSE;
		}
		// Else compare the char.
		else if(c1 != c2)
			return FALSE;
	}
	// The same.
	return TRUE;
}

// Compare a file name created with GetShortFileName()
// to the fileNameTitle param. 
// Profile release - 22 ms for 851,808 tests.
BOOL CVideo::CompareFileNames(const CString & fileNameTitle)
{
	//
	// This avoids the call to GetShortFileName() for performance.
	//

	int len1 = mp4.GetLength() - 4;				// less .mp4
	int len2 = fileNameTitle.GetLength() - 4;	// less .mp4
	TCHAR c1, c2;

	// FileName param must be shorter or equal to mp4.
	if(len1 < len2)
		return FALSE;

	// Ensure bounds.
	if(len1 < 1 || len2 < 1)
		return FALSE;

	// Loop chars backwards.
	for(int i = 1; i <= len2; i++)
	{
		c1 = mp4.GetAt(len1 - i);
		c2 = fileNameTitle.GetAt(len2 - i);	// len2 - len2 = element 0
		if(c1 != c2)
			return FALSE;
	}
	// The same.
	return TRUE;
}

// Compare two mp4 member strings.
// Return:
//   0 if identical.
//  -1 when first char from end is less than
//   1 when first char from end is greater than
//
// Called from:
//  CTopic::VideoInstancesMp4()
//	CHistory::HistoryExists()
//	CHistory::VideoMp4Exists
//	CHistory::NumUniqueHistoryItems()
//	CHistoryArray::SortHistoryMp4Count()
//
int CVideo::CompareMp4Strings(const CString & mp4Str)
{
	//
	// CString operator::== does a fine job, but all the mp4 members 
	// have about 44 chars in common up front:
	//
	//	http://s3.amazonaws.com/KA-youtube-converted/l-QFT7XNeb4.mp4/l-QFT7XNeb4.mp4"
	//
	// which are compared before getting to the l-QFT7XNeb4 chars.
	//
	// Here we compare backwards the chars between .mp4 and the first /.
	//

	int len = mp4.GetLength() - 4;		// less .mp4
	int len2 = mp4Str.GetLength() - 4;	// less .mp4

	// Ensure at least one char to test.
	if(len < 1 || len2 < 1)
		return -1;


	TCHAR c1, c2;

	// Loop chars backwards.
	for(int i = len - 1, i2 = len2 - 1; i >= 0, i2 >= 0; i--, i2--)
	{
		c1 = mp4.GetAt(i);
		c2 = mp4Str.GetAt(i2);
		if(c1 < c2)
			return -1;
		if(c1 > c2)
			return 1;

		// Done when back to the first "/" char - /l-QFT7XNeb4.mp4.
		if(c1 == _T('/') || c2 == _T('/'))
		{
			// Error - return -1 ok - see callers
			if(c1 != c2)
				return -1;

			// Ok
			break;
		}
	}
	// The same.
	return 0;
}

// test search
// Get text
BOOL CVideo::GetText(CString & _description, CString & _title, UINT & nChars)
{
	BOOL ret = FALSE;
	if(!description.IsEmpty())
	{
		_description = description;
		nChars += description.GetLength();
		ret = TRUE;
	}
	if(!title.IsEmpty())
	{
		_title = title;
		nChars += title.GetLength();
		ret = TRUE;
	}
	return ret;
}

// test search
// Param search must be lower case.
// Param searchLen is search str's length.
BOOL CVideo::GetText2(CString & buf, const CString & search, 
					  const int & searchLen, UINT & nFound)
{
	BOOL ret = FALSE;
	int start = 0;
	if(!description.IsEmpty())
	{
		buf = description;
		buf.MakeLower();
		start = 0;

		// Note: need only one hit per string. No need to
		// check whole string.

		start = buf.Find(search, start);
		while(start != -1)
		{
			int bufLen = buf.GetLength();

			// If searchStr is not at the begining of the string,
			// ensure a space is before it.
			if(start && buf[start - 1] != _T(' '))
			{
				TCHAR c = buf[start - 1];
				int g = 0;
			}
			// If searchStr is not at the end of the string,
			// ensure a valid char is after it.
			else if(start && 
					start + searchLen < bufLen			&&
					( buf[start + searchLen] != _T(' ') &&
					  buf[start + searchLen] != _T(',') &&
					  buf[start + searchLen] != _T('.') &&
					  buf[start + searchLen] != _T('?') &&
					  buf[start + searchLen] != _T('!') ))
			{
				TCHAR c = buf[start + searchLen];
				int g = 0;
			}
			// Else a match.
			else
				nFound++;

			// Start at end of last found;
			start += searchLen; 
			start = buf.Find(search, start);
			ret = TRUE;
		}
	}
	if(!title.IsEmpty())
	{
		buf = title;
		buf.MakeLower();
		start = 0;

		start = buf.Find(search, start);
		while(start != -1)
		{
			int bufLen = buf.GetLength();

			// If searchStr is not at the begining of the string,
			// ensure a space is before it.
			if(start && buf[start - 1] != _T(' '))
			{
				TCHAR c = buf[start - 1];
				int g = 0;
			}
			// If searchStr is not at the end of the string,
			// ensure a valid char is after it.
			else if(start && 
					start + searchLen < bufLen			&&
					( buf[start + searchLen] != _T(' ') &&
					  buf[start + searchLen] != _T(',') &&
					  buf[start + searchLen] != _T('.') &&
					  buf[start + searchLen] != _T('?') &&
					  buf[start + searchLen] != _T('!') ))
			{
				TCHAR c = buf[start + searchLen];
				int g = 0;
			}
			// Else a match.
			else
				nFound++;
			// Start at end of last found;
			start += searchLen; 
			start = buf.Find(search, start);
			ret = TRUE;
		}
	}
	return ret;
}



void CVideo::Read(CArchive & ar)
{
	ar >> topic;
	ar >> date_added;
	ar >> description;
	ar >> mp4;
	ar >> duration;
	ar >> title; 
}
void CVideo::Write(CArchive & ar)
{
	ar << topic;
	ar << date_added;
	ar << description;
	ar << mp4;
	ar << duration;
	ar << title; 
}

///////////////////////////////////////////////////////////////////
// CTopics

CTopic::CTopic()
{
/*
	int sz;
	sz = sizeof(CVideo);		// 28 empty
	sz = sizeof(CTopic);		// 52 empty
	sz = sizeof(CTopicArray);	// 20 empty
	sz = sizeof(CVideoArray);	// 20 empty
	sz = sizeof(CString);		// 4  empty
*/
	//
	// per task manager - 2036 k used post loading full library - release.

}
CTopic::~CTopic()
{
}
CTopic::CTopic(const CTopic & topic)
{
	Copy(topic);
}

void CTopic::operator=(const CTopic & topic)
{
	Copy(topic);
}
BOOL CTopic::operator==(const CTopic & topic)
{
	// Note CString == operator is case sensitive.

	if(treeTopic == topic.treeTopic &&
	   parentTreeTopic == topic.parentTreeTopic &&
	   title == topic.title &&
	   description == topic.description)
	{
		int szThis = videos.GetSize();
		int szParam = topic.videos.GetSize();
		if(szThis == szParam)
		{
			// Check each video.
			for(int i = 0; i < szThis; i++)
			{
				if(videos[i] != topic.videos[i])
					return FALSE;
			}
		}
		// Equal.
		return TRUE;
	}
	// No go.
	return FALSE;
}
BOOL CTopic::operator!=(const CTopic & topic)
{
	return !(*this == topic); 
}
void CTopic::Copy(const CTopic & topic)
{
	treeTopic		= topic.treeTopic;
	parentTreeTopic = topic.parentTreeTopic;
	title			= topic.title;
	description		= topic.description;
	videos.Copy(topic.videos);
	childTopics.Copy(topic.childTopics);
}

void CTopic::Zero()
{
	treeTopic.Empty();
	parentTreeTopic.Empty();
	title.Empty();
	description.Empty();

	childTopics.RemoveAll();
	childTopics.SetSize(0, 8);
	videos.RemoveAll();
	videos.SetSize(0, 32);
}


// Returns -1 if nonexistant, else the topics element.
BOOL CTopic::GetTopicRef(CTopic & topic, const CString & treeTopic)
{
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(childTopics[i].treeTopic == treeTopic)
		{
			topic = childTopics[i];
			return TRUE;
		}
	}
	return FALSE;
}

// Get a CTopic per its Title str (not treeTopic str).
// old - use GetChildTopicByPos
BOOL CTopic::GetChildTopicOld(CTopic & topic, CStringArray & nodeTitleList, int listElmt)
{
	// Ensure valid list element.
	int listSz = nodeTitleList.GetSize();
	if(listElmt >= listSz)
		return FALSE;

	// Check this Topic's children for a Title match.
	int sz = childTopics.GetSize();

	for(int i = 0; i < sz; i++)
	{
		if( childTopics[i].title == nodeTitleList[listElmt] )
		{
			// If further child topics.
			if(listElmt + 1 < listSz)
			{
				return childTopics[i].GetChildTopicOld(topic, nodeTitleList, listElmt + 1);
			}

			// If this is the final item in the list, it's our topic.
			else if(listElmt + 1 == listSz)
			{
				topic = childTopics[i];
				return TRUE;
			}
			// Error.
			return FALSE;
		}
	}
	return FALSE;
}

// Fix 9/13/2017 - duplicate topic title fix
// Get a CTopic per its Title str and Topic Tree pos.
BOOL CTopic::GetChildTopicByPos(CTopic & topic, CNodeArray & nodeAr, int nodeArElmt)
{
	// Ensure valid node array elmt param.
	int nodeArSz = nodeAr.GetSize();
	if(nodeArElmt >= nodeArSz)
		return FALSE;

	// Ensure valid node.
	if(!nodeAr[nodeArElmt].IsValid())
		return FALSE;

	// Get the child topic tree index.
	int index = nodeAr[nodeArElmt].m_arIndex;
	//
	// Check for valid child topic tree index.
	int sz = childTopics.GetSize();
	if(index >= sz)
		return FALSE;

	// Check if child titles match.
	if(nodeAr[nodeArElmt].m_title != childTopics[index].title)
		return FALSE;

	// If at the last nodeAr item, we are done.
	if(nodeArElmt + 1 == nodeArSz)
	{
		topic = childTopics[index];
		return TRUE;
	}
	// Else go down the list.
	else
	{
		return childTopics[index].GetChildTopicByPos(topic, nodeAr, nodeArElmt + 1);
	}
	// Error
	return FALSE;
}

// Get a topic per its treeTopic str.
BOOL CTopic::GetTopicByTreeTopic(CTopic & topic, const CStringArray & nodeList, int listElmt)
{
	// Ensure valid list element.
	int listSz = nodeList.GetSize();
	if(listElmt >= listSz)
		return FALSE;

	// Get the root.
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(childTopics[i].treeTopic == nodeList[listElmt])
		{
			// If the last topic in the list, this is the Topic.
			if(listElmt + 1 == listSz)
			{
				topic = childTopics[i];
				return TRUE;
			}
			// Else check children.
			return childTopics[i].GetTopicByTreeTopic(topic, nodeList, listElmt + 1);
		}
	}
	// No go.
	return FALSE;
}

// Update a topic with newTopicData.
// Note nodeList is treeTopic strings.
BOOL CTopic::UpdateTopic(CTopic & newTopicData, CStringArray & nodeList, 
						 int listElmt, BOOL stringsOnly)
{
	// Ensure valid list element.
	int listSz = nodeList.GetSize();
	if(listElmt >= listSz)
		return FALSE;

	// Get the child topic.
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(childTopics[i].treeTopic == nodeList[listElmt])
		{
			// If the last topic in the list, this is the Topic.
			if(listElmt + 1 == listSz)
			{
				// Update.
				if(childTopics[i].treeTopic == newTopicData.treeTopic &&
				   childTopics[i].parentTreeTopic == newTopicData.parentTreeTopic)
				{
					if(stringsOnly)
					{
						if(!newTopicData.title.IsEmpty())
						{
							childTopics[i].title = newTopicData.title;
						}
						if(!newTopicData.description.IsEmpty())
							childTopics[i].description = newTopicData.description;
					}
					else
						childTopics[i] = newTopicData;
					return TRUE;
				}
				// Not a match.
				return FALSE;
			}
			// Else check children.
			return childTopics[i].UpdateTopic(newTopicData, nodeList, 
											  listElmt + 1, stringsOnly);
		}
	}
	// No go.
	return FALSE;
}

// Set nodeList param with the path to the treeTopicStr param.
// This is a linear serch through the tree.
BOOL CTopic::GetTopicPath(const CString & treeTopicStr, CStringArray & nodeList)
{
	// If this is the topic.
	if(treeTopic == treeTopicStr)
	{
		nodeList.InsertAt(0, treeTopic);
		return TRUE;
	}
	// Else check the topic's children.
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// If down the list.
		if(childTopics[i].GetTopicPath(treeTopicStr, nodeList))
		{
			// Add this at the head.
			nodeList.InsertAt(0, treeTopic);
			return TRUE;
		}
	}
	// Not in this topic.
	return FALSE;
}


// Sort this Topic and all its children.
void CTopic::Sort(BOOL & sortChild, BOOL & sortVideo)
{
	int sz, i;

	// Sort this Topic's child array.
	if(childTopics.GetSize() && sortChild)
		childTopics.SortTopicsDecend();

	// Videos are apparently pre-ordered, per khan website - 4/5/12
	// Sort this topic's Videos.
	if(sortVideo)
		videos.SortVideoDateAscend();

	// Sort child topics of this Topic's child array.
	sz = childTopics.GetSize();
	for(i = 0; i < sz; i ++)
	{
		if(sortChild)
			childTopics[i].Sort(sortChild, sortVideo);
	}
}

// Add the topicToAdd Child Topic.
//
// The topic to add will be the last topic in the nodesList.
//
// If the child topic to add has parent topics which have not
// yet been added, add those child parents as empty topics. 
void CTopic::AddChildTopics(CTopic & topicToAdd, CStringArray & nodesList, 
						    int nodeElmtToAdd)
{
	// Verify.
	int nodesListSz = nodesList.GetSize();	
	if(nodeElmtToAdd >= nodesListSz)
		return;

	// Child we're adding.
	CString childTopicNodeStr = nodesList[nodeElmtToAdd];

	// Check if the child topic has already been added.
	int childTopicElmnt = HasChildTopic(childTopicNodeStr);

	// If the child topic does not exist.
	if(childTopicElmnt == -1)
	{
		// If this is the last topic in the nodesList, 
		// we're adding the topicToAdd. 
		if(nodeElmtToAdd == nodesListSz - 1)
		{
			childTopics.Add(topicToAdd);
			return;
		}
		// Else add an empty topic.
		else
		{
			CTopic temp;
			temp.parentTreeTopic = nodesList[nodeElmtToAdd - 1];
			temp.treeTopic = nodesList[nodeElmtToAdd];
			temp.title = SetEmptyTopicTitle(temp.treeTopic);

			// Add the empty topic, and save its array element.
			childTopicElmnt = childTopics.Add(temp);

			// Verify valid index.
			if(childTopicElmnt == -1)
			{
				TRACE(_T("ERROR - CTopic::AddChildTopics - temp topic not added\n"));
				return;
			}
		}
	}
	// The existing topic or just added empty topic 
	// may be the parent of subsequent child topics.
	//
	// If another child to add.
	int nodeToAdd = nodeElmtToAdd + 1;

	// Add the next Child Topic node.
	if(nodeToAdd < nodesListSz)
	{
		// Add the child topics
		childTopics[childTopicElmnt].AddChildTopics(topicToAdd,	// last in nodeList.
													nodesList,	// all parents of topicToAdd.
													nodeToAdd);	// current node to add.
	}
	return;
}

BOOL CTopic::VideoExists(const CVideo & vid, const CStringArray & nodeList,
						 int curNode)
{
	// Ensure in bounds.
	int nodeArSz = nodeList.GetSize();
	if(curNode >= nodeArSz)
		return FALSE;

	//dbg
	CString s = nodeList[curNode];

	// CurNode is this topic.
	if(nodeList[curNode] != treeTopic)
		return FALSE;

	// If at the last node. 
	if(curNode == nodeArSz - 1)
	{
		// This should be the vid's topic.
		if(vid.topic != treeTopic)
			return FALSE;

		// Check for video match.
		int nVids = videos.GetSize();

		// Check for the video.
		for(int iVid = 0; iVid < nVids; iVid ++)
		{
			if(videos[iVid].title == vid.title)
				return TRUE;
		}

		// No video match.
		return FALSE;
	}

	// Else check children for a match with the next node.
	curNode ++;
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// Get a ref.
		CTopic & topic = childTopics[i];

		// If at the node.
		if(topic.treeTopic == nodeList[curNode])
			return topic.VideoExists(vid, nodeList, curNode);
	}
	// No topic matches node list. 
	return FALSE;
}

// Count, and store as needed, all videos in this 
// topic and its child topics.
void CTopic::GetTopicVideos(int & nVideos, CVideoArray * videoAr)
{
	int i, sz;

	// If topic has vids.
	sz = videos.GetSize();
	if(sz)
	{
		// Add up.
		nVideos += sz;

		// If storing.
		if(videoAr)
		{
			for(i = 0; i < sz; i ++)
			{
				videoAr->Add(videos[i]);
			}
		}
	}

	// Check all child topics for vids.
	sz = childTopics.GetSize();
	for(i = 0; i < sz; i++)
	{
		childTopics[i].GetTopicVideos(nVideos, videoAr);
	}
}


// Find all instances of a Video Title in this topic and its children.
// If any found, save its Topic in topicsAr.
// This is a complete linear search of the DB.
void CTopic::VideoInstances(const CString & vidTitle, CStringArray & topicsAr)
{
	int i, sz;

	// Check video array.
	sz = videos.GetSize();
	for(i = 0; i < sz; i ++)
	{
		// Should not be more than one per Topic.
		if(videos[i].title == vidTitle)
		{
			topicsAr.Add(treeTopic);
			break;
		}
	}

	// Check all children.
	sz = childTopics.GetSize();
	for(i = 0; i < sz; i++)
	{
		childTopics[i].VideoInstances(vidTitle, topicsAr);
	}
}

// Find all instances of a Video Title in this topic and its children.
// If any found, save the CVideo in videoAr.
// This is a complete linear search of the DB.
void CTopic::VideoInstances(const CString & vidTitle, CVideoArray & videoAr)
{
	int i, sz;

	// Check video array.
	sz = videos.GetSize();
	for(i = 0; i < sz; i ++)
	{
		// Should not be more than one per Topic.
		if(videos[i].title == vidTitle)
		{
			videoAr.Add(videos[i]);
			break;
		}
	}

	// Check all children.
	sz = childTopics.GetSize();
	for(i = 0; i < sz; i++)
	{
		childTopics[i].VideoInstances(vidTitle, videoAr);
	}
}
// Revised above 10/28/13
// Check this Topic for one instance of the vid params mp4.
void CTopic::VideoInstancesMp4(const CVideo & vid, CVideoArray & videoAr)
{
	int i, sz;

	// Check video array.
	sz = videos.GetSize();
	for(i = 0; i < sz; i ++)
	{
		// Should not be more than one per Topic.
		if(videos[i].CompareMp4Strings(vid.mp4) == 0)
		{
			videoAr.Add(videos[i]);
			break;
		}
	}

	// Check all children.
	sz = childTopics.GetSize();
	for(i = 0; i < sz; i++)
	{
		childTopics[i].VideoInstancesMp4(vid, videoAr);
	}
}

// Returns -1 if nonexistant, else the topics element.
int	CTopic::HasChildTopic(const CString & childTreeTopic)
{
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(childTopics[i].treeTopic == childTreeTopic)
			return i;
	}
	return -1;
}

// Set the title for an empty topic.
// Note function duplicated in CTopicTree.
// revised v 0.9.0
CString CTopic::SetEmptyTopicTitle(const CString & treeTopicStr)
{
	CString str = treeTopicStr;
	CString up = _T("Empty");
	if(!str.IsEmpty())
	{
		up = str[0];
		up.MakeUpper();
		str.Delete(0, 1);
		up += str;
	}
	return up;
}

// Count Topics.
int CTopic::NumTopics()
{
	int nTopics = 0;

	// If this topic has videos.
	if(videos.GetSize())
		nTopics ++;

	// Check child topics.
	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i ++)
	{
		// Check for children.
		nTopics += childTopics[i].NumTopics();
	}
	return nTopics;
}

// Delete a topic.
BOOL CTopic::DeleteTopic(const CStringArray & nodesList, int listElmt)
{
	// Ensure in bounds.
	int nodeArSz = nodesList.GetSize();
	if(listElmt >= nodeArSz)
		return FALSE;

	//dbg
	CString s = nodesList[listElmt];

	int sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(childTopics[i].treeTopic == nodesList[listElmt])
		{
			// If listElmt is last element.
			if(listElmt + 1 == nodeArSz)
			{
				TRACE(_T("CTopic DeleteTopic - Removing topic %d\n"), listElmt);
				childTopics.RemoveAt(i);
				return TRUE;
			}
			// Else go to child.
			return childTopics[i].DeleteTopic(nodesList, listElmt + 1);
		}
	}
	return FALSE;
}

// Delete a child topic by pos. 10/04/2017
// Called only from CTopicTree::DeleteTopicByPos().
BOOL CTopic::DeleteChildTopicByPos(const CNodeArray & nodeAr, int nodeArElmt)
{
	// Ensure valid node array elmt param.
	int nodeArSz = nodeAr.GetSize();
	if(nodeArElmt >= nodeArSz)
		return FALSE;

	// Ensure valid node.
	if(!nodeAr[nodeArElmt].IsValid())
		return FALSE;

	// Get the child topic tree index.
	int index = nodeAr[nodeArElmt].m_arIndex;
	//
	// Check for valid child topic tree index.
	int sz = childTopics.GetSize();
	if(index >= sz)
		return FALSE;

	// Check if child titles match.
	if(nodeAr[nodeArElmt].m_title != childTopics[index].title)
		return FALSE;

	// If at the last nodeAr item, we are done.
	if(nodeArElmt + 1 == nodeArSz)
	{
		TRACE(_T("CTopic DeleteChildTopicByPos - Removing topic %d\n"), index);
		childTopics.RemoveAt(index);
		return TRUE;
	}
	// Else go down the list.
	else
	{
		return childTopics[index].DeleteChildTopicByPos(nodeAr, nodeArElmt + 1);
	}
	// Error
	return FALSE;
}



// Print this topic and all its child topics.
void CTopic::PrintTopic(CStdioFile & file, int nTabs)
{
	CString tabStr, newLine, s;
	newLine = _T('\n');
	int tabs = nTabs;
	while(tabs > 0)
	{
		tabStr += _T('\t');
		tabs--;
	}
	// Print title str.
	if(!title.IsEmpty())
	{
		if(nTabs)
			file.WriteString(tabStr);	// Print the tabs
		s = title;
		s += newLine;
		file.WriteString(s);
	}
	// Else print treeTopic str.
	else
	{
		if(nTabs)
			file.WriteString(tabStr);
		s = treeTopic;
		s += newLine;
		file.WriteString(s);
	}

	int sz, i;

	// Print videos.
	CString vid = _T("vid: ");
	//
	// In one tab.
	tabStr += _T('\t');
	sz = videos.GetSize();
	for(i = 0; i < sz; i ++)
	{
		// Tab over.
		file.WriteString(tabStr);
		// Print prefix.
		file.WriteString(vid);
		// Print video title.
		s = videos[i].title;
		s += newLine;
		file.WriteString(s);
	}

	// Print any childTopics.
	sz = childTopics.GetSize();
	for(i = 0; i < sz; i ++)
	{
		childTopics[i].PrintTopic(file, nTabs + 1);
	}
}

////////////////////////////////////
// Search

// 10/04/2017 search
// Search this topic and its children for the term param.
// Term param must be lower case.
// Store hits in the results ar param.
// topicArIndex is the array index this topic is stored in.
// Profile: Release - 21 ms - 26308 hits - 51% time in MakeLower().
BOOL CTopic::SearchTopic(const CString & term, CSearchArray & results,
						 CSearchItem & searchItem, int topicArIndex, 
						 int & nSearched, int & nFound)
{
	BOOL ret = FALSE;
	int iTopicNode = 0, iVideoNode = 0, nRemove = 1;
	CString bufTitle, bufDiscription;

	// Start with this topic.
	//
	// Set topic node index.
	iTopicNode = searchItem.m_nodeIndices.Add(topicArIndex);

	// Get lower.
	bufTitle = title;
	bufDiscription = description;
	bufTitle.MakeLower();
	bufDiscription.MakeLower();

	// Check this topic's strings.
	if(bufTitle.Find(term)		 != -1	||
	   bufDiscription.Find(term) != -1	)
	{
		results.Add(searchItem);
		nFound ++;
		ret = TRUE;
	}
	nSearched ++;

	// Check the topic's videos for a term match
	int sz = videos.GetSize();
	for(int i = 0; i < sz; i++)
	{
		CVideo & video = videos[i];

		// Get lower.
		bufTitle = video.title;
		bufDiscription = video.description;
		bufTitle.MakeLower();
		bufDiscription.MakeLower();


		// Check this topic video's strings.
		if(bufTitle.Find(term)		 != -1 ||
		   bufDiscription.Find(term) != -1 )
		{
			// Set video node index and flag.
			iVideoNode = searchItem.m_nodeIndices.Add(i);
			searchItem.m_isVideo = TRUE;

			// Add to results.
			results.Add(searchItem);

			// Remove video node and flag for next hit.
			nRemove = searchItem.m_nodeIndices.GetSize() - iVideoNode; // not needed here
			searchItem.m_nodeIndices.RemoveAt(iVideoNode, nRemove);
			searchItem.m_isVideo = FALSE;
			nFound ++;
			ret = TRUE;
		}
		nSearched ++;
	}

	// Check the topic's children for a term match.
	sz = childTopics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// Call child topic search.
		if(childTopics[i].SearchTopic(term, results, searchItem, i, nSearched, nFound))
			ret = TRUE;
	}

	// Done with this topic. Remove from nodes list.
	nRemove = searchItem.m_nodeIndices.GetSize() - iTopicNode;
	searchItem.m_nodeIndices.RemoveAt(iTopicNode, nRemove);

	return ret;
}

// 10/04/2017 search
// Get a child topic or video Title and Description strings.
BOOL CTopic::SearchTopicGetStrings(const CSearchItem & searchItem, CString & strTitle, 
								   CString & strDescription, int iCurNode)
{	
	// Check nodes bounds.
	int szNodes = searchItem.m_nodeIndices.GetSize();
	if(iCurNode >= szNodes)
		return FALSE;

	// If this Topic is the last node.
	if(iCurNode + 1 == szNodes)
	{
		// This node cannot be a video -it's a topic.
		if(searchItem.m_isVideo)
			return FALSE;

		// Else return this Topic's strings.
		strTitle = title;
		strDescription = description;
		return TRUE;
	}

	// Else check the next node.
	int iNextNode = iCurNode + 1;

	// If the next node is the last and is a video,
	// it's a video from this Topic.
	if(iNextNode + 1 == szNodes && searchItem.m_isVideo)
	{
		int iVidAr = searchItem.m_nodeIndices[iNextNode];
		int nVideos = videos.GetSize();

		// Check bounds.
		if(iVidAr >= nVideos)
			return FALSE;

		// Get video
		CVideo & video = videos[iVidAr];
		strTitle = video.title;
		strDescription = video.description;
		return TRUE;
	}

	// Else the next node is a child topic.
	else if(iNextNode < szNodes)
	{
		// Get this child TopicTree index.
		int iTreeAr = searchItem.m_nodeIndices[iNextNode];

		// Get child topic tree ar size.
		int szTreeAr = childTopics.GetSize();

		// Check bounds.
		if(iTreeAr >= szTreeAr)
			return FALSE;

		// Get the child Topic.
		CTopic & childTopic = childTopics[iTreeAr];

		// Call
		return childTopic.SearchTopicGetStrings(searchItem, 
												strTitle, strDescription, iNextNode);
	}
	// Problem - should not get here.
	return FALSE;
}

// 10/04/2017 search
// Get search item tree nodes using topic titles.
// Used for display in a list.
BOOL CTopic::SearchTopicGetNodeTitles(const CSearchItem & searchItem, 
									  CString & titlesStr, int iCurNode, int maxLen)
{
	// Check nodes bounds.
	int szNodes = searchItem.m_nodeIndices.GetSize();
	if(iCurNode >= szNodes)
		return FALSE;

	// If this Topic is the last node.
	if(iCurNode + 1 == szNodes)
	{
		// This node cannot be a video -it's a topic.
		if(searchItem.m_isVideo)
			return FALSE;

		// Else return this Topic's strings.
		titlesStr += SearchFormatTitleStr(title, FALSE, maxLen);
		return TRUE;
	}
	// Else add the topic node.
	else
	{
		CString str;
		titlesStr += SearchFormatTitleStr(title, TRUE, maxLen);
	}

	// Else check the next node.
	int iNextNode = iCurNode + 1;

	// If the next node is the last and is a video,
	// it's a video from this Topic.
	if(iNextNode + 1 == szNodes && searchItem.m_isVideo)
	{
		int iVidAr = searchItem.m_nodeIndices[iNextNode];
		int nVideos = videos.GetSize();

		// Check bounds.
		if(iVidAr >= nVideos)
			return FALSE;

		// Get video
		CVideo & video = videos[iVidAr];
		titlesStr += SearchFormatTitleStr(video.title, FALSE, maxLen); // friend
		return TRUE;
	}

	// Else the next node is a child topic.
	else if(iNextNode < szNodes)
	{
		// Get this child TopicTree index.
		int iTreeAr = searchItem.m_nodeIndices[iNextNode];

		// Get child topic tree ar size.
		int szTreeAr = childTopics.GetSize();

		// Check bounds.
		if(iTreeAr >= szTreeAr)
			return FALSE;

		// Get the child Topic.
		CTopic & childTopic = childTopics[iTreeAr];

		// Call
		return childTopic.SearchTopicGetNodeTitles(searchItem, titlesStr, iNextNode, maxLen);
	}
	// Problem - should not get here.
	return FALSE;
}

// 10/04/2017 search
// Get the treeTopic strings and vidio index for a CSearchItem.
//
// Set nodeList param for the Topic location.
//
// Set iVideo param for video index in the topics video array,
// or -1 if not a video search item.
//
// Called from CTopicTree::SearchGetItemPath()
BOOL CTopic::SearchTopicGetItemPath(const CSearchItem & searchItem, CStringArray & nodeList,
									int & iVideo, int iCurNode)
{
	// Check nodes bounds.
	int szNodes = searchItem.m_nodeIndices.GetSize();
	if(iCurNode >= szNodes)
		return FALSE;

	// If this Topic is the last node.
	if(iCurNode + 1 == szNodes)
	{
		// This node cannot be a video - it's a topic.
		if(searchItem.m_isVideo)
			return FALSE;

		// Else save the tree node path.
		nodeList.Add(treeTopic);
		// Ensure no video.
		iVideo = -1;
		return TRUE;
	}
	// Else add the topic node.
	else
	{
		nodeList.Add(treeTopic);
	}

	// Check the next node.
	int iNextNode = iCurNode + 1;

	// If the next node is the last and is a video,
	// it's a video from this Topic.
	if(iNextNode + 1 == szNodes && searchItem.m_isVideo)
	{
		int iVidAr = searchItem.m_nodeIndices[iNextNode];
		int nVideos = videos.GetSize();

		// Check bounds.
		if(iVidAr >= nVideos)
			return FALSE;

		// Topic node was added to nodeList above.
		// Set the video ar index for that topic.
		iVideo = iVidAr;
		return TRUE;
	}

	// Else the next node is a child topic.
	else if(iNextNode < szNodes)
	{
		// Get this child TopicTree index.
		int iTreeAr = searchItem.m_nodeIndices[iNextNode];

		// Get child topic tree ar size.
		int szTreeAr = childTopics.GetSize();

		// Check bounds.
		if(iTreeAr >= szTreeAr)
			return FALSE;

		// Get the child Topic.
		CTopic & childTopic = childTopics[iTreeAr];

		// Call child topic.
		return childTopic.SearchTopicGetItemPath(searchItem, nodeList,
												 iVideo, iNextNode);
	}
	// Problem - should not get here.
	return FALSE;
}

// old - delete
// Check if a video file name, created with GetShortFileNameTitle(),
// exists in this Topic.
// Performs a linear search.
// Profile: 30 ms for 511 files found.
BOOL CTopic::VideoFileUsedTitleOld(const CString & fileName, CVideo * pVideo)
{
	// Check videos in this Topic.
	int sz = videos.GetSize();
	CString name; 
	for(int i = 0; i < sz; i++)
	{
	/*	if(videos[i].CompareFileNamesTitle(fileName))	// 30 ms for 511 files found.
		{
			// Copy the video to param as needed.
			if(pVideo)
				*pVideo = videos[i];
			return TRUE;
		}*/
	}

	// Check child Topics.
	sz = childTopics.GetSize();
#if VS2005
	for(int i = 0; i < sz; i++)	// vs2005
#else
	for(i = 0; i < sz; i++)	// vc6
#endif
	{
	//	if(childTopics[i].VideoFileUsedTitle(fileName, pVideo))
	//		return TRUE;
	}

	// Nothing.
	return FALSE;
}

// Check if a video file name, created with GetShortFileName(),
// exists in this Topic. Performs a linear search.
BOOL CTopic::VideoFileUsed(const CString & fileName, CVideo * pVideo)
{
	// Check videos in this Topic. about 100x slower.
/*	int sz = videos.GetSize();
	CString name; 
	for(int i = 0; i < sz; i++)
	{
		if(videos[i].GetShortFileName(name))
		{
			if(name == fileName)
			{
				// Copy the video to param as needed.
				if(pVideo)
					*pVideo = videos[i];
				return TRUE;
			}
		}
	}*/

	// Check videos in this Topic.
	int sz = videos.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(videos[i].CompareFileNames(fileName))
		{
			// Copy the video to param as needed.
			if(pVideo)
				*pVideo = videos[i];
			return TRUE;
		}
	}

	// Check child Topics.
	sz = childTopics.GetSize();
#if VS2005
	for(int i = 0; i < sz; i++)	// vs2005
#else
	for(i = 0; i < sz; i++)			// vc6
#endif
	{
		if(childTopics[i].VideoFileUsed(fileName, pVideo))
			return TRUE;
	}
	// Nothing.
	return FALSE;
}


void CTopic::Read(CArchive & ar)
{
	ar >> treeTopic;
	ar >> parentTreeTopic;
	ar >> description;
	ar >> title;

	// Videos.
	int sz;
	ar >> sz;
	if(sz)
	{
		CVideo video;
		for(int i = 0; i < sz; i ++)
		{
			video.Read(ar);
			videos.Add(video);
			video.Zero();
		}
	}

	// Child Topics.
	ar >> sz;
	if(sz)
	{
		CTopic topic;
		for(int i = 0; i < sz; i ++)
		{
			topic.Read(ar);
			childTopics.Add(topic);
			topic.Zero();
		}
	}
}
void CTopic::Write(CArchive & ar)
{
	ar << treeTopic;
	ar << parentTreeTopic;
	ar << description;
	ar << title;

	// Videos.
	int i, sz = videos.GetSize();
	ar << sz;
	for(i = 0; i < sz; i++)
	{
		videos[i].Write(ar);
	}

	// Child Topics.
	sz = childTopics.GetSize();
	ar << sz;
	for(i = 0; i < sz; i++)
	{
		childTopics[i].Write(ar);
	}
}


////////////////////////////////////////////////////////////////
// class CTopicTree

CTopicTree::CTopicTree()
{
	Zero();
}
CTopicTree::~CTopicTree()
{
}

void CTopicTree::Zero() 
{
	m_topics.RemoveAll();
	m_topics.SetSize(0, 16);	// just roots

	// DB stats - consider for later versions - not currently serialized.
//	m_nTopics = 0;
//	m_nVideos = 0;
//	m_creationTime = GetDefaultTime();
}

/*
// test - consider
void CTopicTree::SetStats()
{
//	m_nTopics = NumTopics();
//	m_nVideos = 0; //NumVideos(); todo
//	m_creationTime = CTime::GetCurrentTime();
}
void CTopicTree::GetStats(int & nTopics, int & nVideos, CTime & creationTime)
{
//	nTopics = m_nTopics;
//	nVideos = m_nVideos;
//	creationTime = m_creationTime;
}
*/

// Add the topicToAdd Topic.
//
// The topic to add will be the last topic in the nodesList.
//
// If the topic to add has parent topics which have not
// yet been added, add those parents as empty topics. 
BOOL CTopicTree::AddTopic(CTopic & topicToAdd, CStringArray & nodesList)
{
	// test fix empty topics.
	//
	// Must have a tree topic and title, and 
	// have either videos or sub topics.
	// Otherwise we get an "Empty Topic".
	int nVids = topicToAdd.videos.GetSize();
	int nSubTopics = topicToAdd.childTopics.GetSize();	
	if(topicToAdd.treeTopic.IsEmpty()	||
	   topicToAdd.title.IsEmpty()		||
	   ( nVids == 0 && nSubTopics == 0 ) )
	{
		TRACE(_T("CTopicTree AddTopic - discarding empty topic %s\n"), topicToAdd.title);

		// Check for vids being discarded.
		if(nVids)
			TRACE(_T("Empty topic has %d videos\n"), nVids);

		return FALSE;
	}

	// Ensure at least one node.
	int nodesListSz = nodesList.GetSize();
	if(!nodesListSz)
		return FALSE;

	// Find the existing Root Topic array element, if it exists.
	int rootElmt = -1;
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].treeTopic == nodesList[0])
		{
			rootElmt = i;
			break;
		}
	}
	// If none, add an empty Root Node.
	if(rootElmt == -1)
	{
		// If only one topic to add, we are done.
		if(nodesListSz == 1)
		{
			rootElmt = m_topics.Add(topicToAdd);			
	//		m_nTopics ++; // dbg
			return TRUE;
		}
		// Else add an empty Root Topic. Note topic.parentTreeTopic is "".
		else
		{
			CTopic topic;
			topic.parentTreeTopic = _T("");
			topic.treeTopic = nodesList[0];
			topic.title = SetEmptyTopicTitle(topic.treeTopic);

			// Append root node and save its array element.
			rootElmt = m_topics.Add(topic);

			// Verify valid index.
			if(rootElmt == -1)
			{
				TRACE(_T("ERROR - CTopicTree AddTopic - root element not added\n"));
				return FALSE;
			}
		}
	}
	// If an existing Root and only one topic to add, 
	// we may be adding videos to the Root.
	if(rootElmt != -1 && nodesListSz == 1)
	{
		// If new topic has videos only.
		if(nVids && !nSubTopics)
		{
			// If root contains no vids.
			if(!m_topics[rootElmt].videos.GetSize())
			{
				m_topics[rootElmt].videos.Copy(topicToAdd.videos);
			}
			return TRUE;
		}		
	}

	// Else it's a child of an existing Root node.
	if(nodesListSz > 0)//tested already above
	{
		// Add root topic's children, starting with #1.
		m_topics[rootElmt].AddChildTopics(topicToAdd, nodesList, 1);
	}

	return TRUE;
}

// Get a topic per its Title str (not its treeTopic str).
// old - use GetTopicByPos()
BOOL CTopicTree::GetTopicOld(CTopic & topic, CStringArray & nodeTitleList)
{
	int nodeArSz = nodeTitleList.GetSize();
	if(!nodeArSz)
		return FALSE;

	// Get the root.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].title == nodeTitleList[0])
		{
			// If only one topic in the list.
			if(nodeArSz == 1)
			{
				topic = m_topics[i];
				return TRUE;
			}
			// Else go down the list.
			else if(m_topics[i].GetChildTopicOld(topic, nodeTitleList, 1))
				return TRUE;
			break;
		}
	}
	// No go.
	return FALSE;
}

// Fix 9/13/2017 - duplicate topic title fix
// Get a topic per its Title str and Topic Tree array pos.
BOOL CTopicTree::GetTopicByPos(CTopic & topic, CNodeArray & nodeAr)
{
	// Must have nodes.
	int nodeArSz = nodeAr.GetSize();
	if(!nodeArSz)
		return FALSE;

	// Ensure valid first node.
	if(!nodeAr[0].IsValid())
		return FALSE;

	// Get the root array.
	int sz = m_topics.GetSize();

	// The first nodeAr item is our topic tree root index.
	int index = nodeAr[0].m_arIndex;
	//
	// Ensure valid root index.
	if(index >= sz)
		return FALSE;

	// Ensure root titles match.
	if(nodeAr[0].m_title != m_topics[index].title)
		return FALSE;

	// If only one node.
	if(nodeArSz == 1)
	{
		topic = m_topics[index];
		return TRUE;
	}
	// Else go down the list.
	else if(m_topics[index].GetChildTopicByPos(topic, nodeAr, 1))
		return TRUE;

	// No go.
	return FALSE;
}

// Get a topic per its treeTopic str.
BOOL CTopicTree::GetTopicByTreeTopic(CTopic & topic, CStringArray & nodeList)
{
	int nodeArSz = nodeList.GetSize();
	if(!nodeArSz)
		return FALSE;

	// Get the root.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].treeTopic == nodeList[0])
		{
			// If only one topic in the list.
			if(nodeArSz == 1)
			{
				topic = m_topics[i];
				return TRUE;
			}
			// Else check children.
			return m_topics[i].GetTopicByTreeTopic(topic, nodeList, 1);
		}
	}
	// No go.
	return FALSE;
}

// Update existing Topic with newTopic param data.
// Note nodeList is treeTopic strings.
BOOL CTopicTree::UpdateTopic(CTopic & newTopicData, CStringArray & nodeList, BOOL stringsOnly)
{
	int nodeArSz = nodeList.GetSize();
	if(!nodeArSz)
		return FALSE;

	// Get the root.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].treeTopic == nodeList[0])
		{
			// If only one topic in the list.
			if(nodeArSz == 1)
			{
				// Update.
				if(m_topics[i].treeTopic == newTopicData.treeTopic &&
				   m_topics[i].parentTreeTopic == newTopicData.parentTreeTopic)
				{
					if(stringsOnly)
					{
						if(!newTopicData.title.IsEmpty())
							m_topics[i].title = newTopicData.title;
						if(!newTopicData.description.IsEmpty())
							m_topics[i].description = newTopicData.description;
					}
					else
						m_topics[i] = newTopicData;
					return TRUE;
				}
				// Not a match.
				return FALSE;
				
			}
			// Else check children.
			return m_topics[i].UpdateTopic(newTopicData, nodeList, 1, stringsOnly);
		}
	}
	// No go.
	return FALSE;
}

// Set nodeList param with the path to the treeTopicStr param.
// This is a linear search through the tree.
BOOL CTopicTree::GetTopicPath(const CString & treeTopicStr, CStringArray & nodeList)
{
	// Ensure empty.
	nodeList.RemoveAll();

	// Search each root's tree for the topic.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].GetTopicPath(treeTopicStr, nodeList))
			return TRUE;
	}
	// Problem if a valid topic
	return FALSE;
}

// Check if a video exists in the khan db.
// Returns false if the CVideo's topic or 
// title are invalid.
BOOL CTopicTree::VideoExists(const CVideo & vid)
{
	// Validate.
	if(vid.topic.IsEmpty())
		return FALSE;
	if(vid.title.IsEmpty())
		return FALSE;

	// Get the path to the video's topic - linear search.
	CStringArray nodeList;
	if(!GetTopicPath(vid.topic, nodeList))
		return FALSE;

	// Validate.
	int nodeArSz = nodeList.GetSize();
	if(!nodeArSz)
		return FALSE;

	// Search roots for nodeList[0].
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// Get a ref.
		CTopic & topic = m_topics[i];

		// If at the node.
		if(topic.treeTopic == nodeList[0])
			return topic.VideoExists(vid, nodeList, 0);
	}
	// No root topic matches first node. 
	return FALSE;
}

// Count, and store as needed in vidoAr param, all videos 
// and all child topic videos in the topic param.
//
// Note this counts and stores duplicate videos 
// listed in multiple topics. 
int CTopicTree::GetTopicVideos(CTopic & topic, CVideoArray * videoAr)
{
	// Results for math:
	//	2110 total
	//	411 dups
	//	1699 unique vids

	int nVids = 0;

	// If storing vids, set up array.
	if(videoAr)
	{
		videoAr->RemoveAll();
		videoAr->SetSize(0, 512);	// 3000 vids  7 root topics
	}

	// Get the vids.
	topic.GetTopicVideos(nVids, videoAr);
	return nVids;
}


// Do a complete linear search of the db for all instances
// of the video Title, See Profile note below.
//
// Save all treeTopics for which the Video is listed under
// in the topics array.
//
// Return false if no title matches.
BOOL CTopicTree::VideoInstances(const CString & vidTitle, CStringArray & topicsAr)
{
	// Ensure.
	if(vidTitle.IsEmpty())
		return FALSE;

	// Zero.
	topicsAr.RemoveAll();
	topicsAr.SetSize(0, 4);

	// Search each root completely.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		m_topics[i].VideoInstances(vidTitle, topicsAr);
	}

	// If found.
	sz = topicsAr.GetSize();
	if(sz)
		return TRUE;

	// None. 
	return FALSE;
}

// Profiling VideoInstances() - release - 21 searches - 1.046 MS
//
// Do a complete linear search of the db for all instances
// of a CVideo with the video Title, See overload above.
//
// Save all CVideos instances in the video array.
//
// Return false if no title matches.
BOOL CTopicTree::VideoInstances(const CString & vidTitle, CVideoArray & videoAr)
{
	// Ensure.
	if(vidTitle.IsEmpty())
		return FALSE;

	// Zero.
	videoAr.RemoveAll();
	videoAr.SetSize(0, 4);

	// Search each root completely.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		m_topics[i].VideoInstances(vidTitle, videoAr);
	}

	// If found.
	sz = videoAr.GetSize();
	if(sz)
		return TRUE;

	// None. 
	return FALSE;
}
// Revised above 10/28/13
// Check enite db for instance of the vid param's mp4.
BOOL CTopicTree::VideoInstancesMp4(const CVideo & vid, CVideoArray & videoAr)
{
	// Ensure.
	if(vid.title.IsEmpty() || vid.mp4.IsEmpty() )
		return FALSE;

	// Zero.
	videoAr.RemoveAll();
	videoAr.SetSize(0, 4);

	// Search each root completely.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		m_topics[i].VideoInstancesMp4(vid, videoAr);
	}

	// If found.
	sz = videoAr.GetSize();
	if(sz)
		return TRUE;

	// None. 
	return FALSE;
}

// Set the title for an empty topic.
// Note function duplicated in CTopic.
// revised v 0.9.0
CString CTopicTree::SetEmptyTopicTitle(const CString & treeTopicStr)
{
	CString str = treeTopicStr;
	CString up = _T("Empty");
	if(!str.IsEmpty())
	{
		up = str[0];
		up.MakeUpper();
		str.Delete(0, 1);
		up += str;
	}
	return up;
}

// Copy all tree Nodes from tree param to this.
void CTopicTree::Copy(CTopicTree & tree)
{
	m_topics.Copy(tree.m_topics);

	return;

	// test valid
	if(m_topics.GetSize() != tree.m_topics.GetSize())
		return;
	CTopic topicDest, topicSrc;
	for(int i = 0; i < m_topics.GetSize(); i++)
	{
		topicDest = m_topics[i];
		topicSrc = tree.m_topics[i];

		if(m_topics[i].childTopics.GetSize() != 
		   tree.m_topics[i].childTopics.GetSize())
			return;

		for(int ii = 0; ii < m_topics[i].childTopics.GetSize(); ii++)
		{
			topicDest = m_topics[i].childTopics[ii];
			topicSrc = tree.m_topics[i].childTopics[ii];
		}
	}
}

// Sort all topics and videos.
void CTopicTree::Sort(BOOL & sortRoot, BOOL & sortChild, BOOL & sortVideo)
{
	// Sort the root topic array.
	if(sortRoot)
		m_topics.SortTopicsDecend();

	// Sort child topic arrays.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		m_topics[i].Sort(sortChild, sortVideo);
	}
}

int	CTopicTree::NumTopics() 
{
	// Count Topics with videos only.
	int nTopics = 0;

	// Lopp 7 or so Roots.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i ++)
	{
		// Check this root topic for videos
		// and all its child topics for videos.
		nTopics += m_topics[i].NumTopics();
	}
	return nTopics;
}


// Delete a topic. Note nodesList contains treeTopic 
// strings, not Title strings.
BOOL CTopicTree::DeleteTopic(CStringArray & nodesList)
{
	// Ensure valid list.
	int listSz = nodesList.GetSize();
	if(!listSz)
		return FALSE;

	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].treeTopic == nodesList[0])
		{
			// If nodes list has only one node,
			// the Topic to delete is this Root.
			if(listSz == 1)
			{
				TRACE(_T("CTopicTree DeleteTopic - Removing ROOT Topic! \n"));
				m_topics.RemoveAt(i);
				return TRUE;
			}
			// Else check children.
			return m_topics[i].DeleteTopic(nodesList, 1);
		}
	}
	return FALSE;
}

// Delete a topic by pos - uses CNodeArray Title strings and indices. // 10/04/2017
BOOL CTopicTree::DeleteTopicByPos(const CNodeArray & nodeAr)
{
	// Must have nodes.
	int nodeArSz = nodeAr.GetSize();
	if(!nodeArSz)
		return FALSE;

	// Ensure valid first node.
	if(!nodeAr[0].IsValid())
		return FALSE;

	// Get the root array.
	int sz = m_topics.GetSize();

	// The first nodeAr item is our topic tree root index.
	int index = nodeAr[0].m_arIndex;
	//
	// Ensure valid root index.
	if(index >= sz)
		return FALSE;

	// Ensure root titles match.
	if(nodeAr[0].m_title != m_topics[index].title)
		return FALSE;

	// If only one node.
	if(nodeArSz == 1)
	{
		TRACE(_T("CTopicTree DeleteTopicByPos - Removing ROOT Topic index %d! \n"), index);
		m_topics.RemoveAt(index);
		return TRUE;
	}
	// Else go down the list.
	else if(m_topics[index].DeleteChildTopicByPos(nodeAr, 1))
		return TRUE;

	// No go.
	return FALSE;
}

// old - delete
// Check if a video file name, created with GetShortFileNameTitle(),
// exists in the DB.
// Performs a complete linear search.
BOOL CTopicTree::VideoFileUsedTitleOld(const CString & fileName, CVideo * pVideo)
{
	// Check videos in each root Topic.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
	//	if(m_topics[i].VideoFileUsedTitle(fileName, pVideo))
	//		return TRUE;
	}
	// Nothing.
	return FALSE;
}

// Check if a video file name, created with GetShortFileName(),
// exists in the DB. Performs a complete linear search.
BOOL CTopicTree::VideoFileUsed(const CString & fileName, CVideo * pVideo)
{
	// Check videos in each root Topic.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		if(m_topics[i].VideoFileUsed(fileName, pVideo))
			return TRUE;
	}
	// Nothing.
	return FALSE;
}

void CTopicTree::PrintTree(CString filePath)
{
	// dbg
	filePath = _T("C:\\KhanLib.txt");

	// File for writing.
	CStdioFile file;
	CFileException e;
	if(!file.Open(filePath, CFile::modeCreate | 
							CFile::modeWrite  |
							CFile::typeText, &e))
	{
		TCHAR szError[1024];
		e.GetErrorMessage(szError, 1024);
		CString er;
		er.Format(_T("Couldn't open file: %s"), szError);
		TRACE(er);
		return;
	}

	// Print each Root, which in turn prints all 
	// of its children.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		m_topics[i].PrintTopic(file);
	}

	// Clean up.
	file.Close();
}

// 10/04/2017 search
// Term param is a word/string to search for.
// If the term is found in a topic or video title/description,
// it is added to the CSearchItems param.
// Called from CSearchDialog::OnSearchButton().
// Profile release: 15.164 ms - 26334 hits - 52% time in MakeLower().
BOOL CTopicTree::Search(const CString & term, CSearchArray & results,
						int & nSearched, int & nFound)
{
	BOOL ret = FALSE;
	CSearchItem searchItem;
	CString bufTerm = term;
	bufTerm.MakeLower();

	// Start from each root.
	int sz = m_topics.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// Limit to about 2000 results.
		if(results.GetSize() > 2000)
			break;

		// Zero for new root topic.
		searchItem.Zero();

		// Call child topic search.
		if(m_topics[i].SearchTopic(bufTerm, results, searchItem, i, nSearched, nFound))
			ret = TRUE;

		// dbg
		int nItems = results.GetSize();
		int g = 0;
	}
	return ret;
}

// 10/04/2017 search
// Get a topic or videos Title and Description strings.
BOOL CTopicTree::SearchGetStrings(const CSearchItem & searchItem, CString & strTitle, 
								  CString & strDescription)
{
	// Get num nodes.
	int szNodes = searchItem.m_nodeIndices.GetSize();
	if(!szNodes)
		return FALSE;

	// Single node cannot be a video.
	if(szNodes == 1 && searchItem.m_isVideo)
		return FALSE;

	// Start at index 0 for the Root.
	int iNode = 0;

	// Get TopicTree index.
	int iTreeAr = searchItem.m_nodeIndices[iNode];

	// Get tree ar size - for roots.
	int szTreeAr = m_topics.GetSize();

	// Check bounds.
	if(iTreeAr >= szTreeAr)
		return FALSE;

	// Get the root Topic.
	CTopic & topic = m_topics[iTreeAr];

	// If this is the last node - not a video for roots.
	if(iNode + 1 == szNodes)
	{
		strTitle = topic.title;
		strDescription = topic.description;
		return TRUE;
	}

	// Set next node.
	int iNextNode = iNode + 1;

	// If this is the last node and a video.
	if(iNextNode + 1 == szNodes && searchItem.m_isVideo)
	{
		int iVidAr = searchItem.m_nodeIndices[iNextNode];
		int nVideos = topic.videos.GetSize();

		// Check bounds.
		if(iVidAr >= nVideos)
			return FALSE;

		// Get video
		CVideo & video = topic.videos[iVidAr];
		strTitle = video.title;
		strDescription = video.description;
		return TRUE;
	}

	// Else node is a child topic.
	else if(iNextNode < szNodes)
	{
		int nChildTopics = topic.childTopics.GetSize();

		// Get ar index.
		int arIndex = searchItem.m_nodeIndices[iNextNode];
		// Check bounds.
		if(arIndex >= nChildTopics)
			return FALSE;
		// Get a ref
		CTopic & childTopic = topic.childTopics[arIndex];
		// Call
		return childTopic.SearchTopicGetStrings(searchItem, strTitle, strDescription, iNextNode);
	}
	// Problem - should not get here
	return FALSE;
}

// 10/04/2017 search
// Get a topic or video tree nodes using the topic or video Title
// truncated to 5 chars and separated by a backward slash.
// Used for display in a list.
BOOL CTopicTree::SearchGetNodeTitles(const CSearchItem & searchItem, CString & titlesStr,
									 int maxLen)
{
	// Zero param.
	titlesStr = _T("");

	// Get num nodes.
	int szNodes = searchItem.m_nodeIndices.GetSize();
	if(!szNodes)
		return FALSE;

	// Single node cannot be a video.
	if(szNodes == 1 && searchItem.m_isVideo)
		return FALSE;

	// Truncate str to
	int max = 8;

	// Start at index 0 for the Root.
	int iNode = 0;

	// Get TopicTree index.
	int iTreeAr = searchItem.m_nodeIndices[iNode];

	// Get tree ar size - for roots.
	int szTreeAr = m_topics.GetSize();

	// Check bounds.
	if(iTreeAr >= szTreeAr)
		return FALSE;

	// Get the root Topic.
	CTopic & topic = m_topics[iTreeAr];

	// If this is the last node - not a video for roots.
	if(iNode + 1 == szNodes)
	{
		titlesStr += SearchFormatTitleStr(topic.title, FALSE, maxLen);
		return TRUE;
	}
	// Else add the topic node.
	else
	{
		titlesStr += SearchFormatTitleStr(topic.title, TRUE, maxLen);
	}

	// Set next node.
	int iNextNode = iNode + 1;

	// If this is the last node and a video.
	if(iNextNode + 1 == szNodes && searchItem.m_isVideo)
	{
		int iVidAr = searchItem.m_nodeIndices[iNextNode];
		int nVideos = topic.videos.GetSize();

		// Check bounds.
		if(iVidAr >= nVideos)
			return FALSE;

		// Get video
		CVideo & video = topic.videos[iVidAr];
		titlesStr += SearchFormatTitleStr(video.title, FALSE, maxLen);
		return TRUE;
	}

	// Else node is a child topic.
	else if(iNextNode < szNodes)
	{
		int nChildTopics = topic.childTopics.GetSize();

		// Get ar index.
		int arIndex = searchItem.m_nodeIndices[iNextNode];
		// Check bounds.
		if(arIndex >= nChildTopics)
			return FALSE;
		// Get a ref
		CTopic & childTopic = topic.childTopics[arIndex];
		// Call
		return childTopic.SearchTopicGetNodeTitles(searchItem, titlesStr, 
												   iNextNode, maxLen);
	}
	// Problem - should not get here
	return FALSE;
}

// Format a path node. Make string maxChars length, or full length if zero,
// Xalled from:
//	CTopicTree::SearchGetNodeTitles()
//	CTopic::SearchTopicGetNodeTitles()
// File scope friend.
CString SearchFormatTitleStr(const CString & title, BOOL addSlash, int maxLen)
{
	CString str;

	// Truncate if greater than max.
	if(maxLen && title.GetLength() > maxLen)
		str = title.Left(maxLen);

	// Else take all - shorter than max.
	else
		str = title;

	// Append \ as needed.
	if(addSlash)
		str += _T("\\");

	// Done
	return str;
}

// 10/04/2017 search
// Get the treeTopic strings and vidio index for a CSearchItem.
//
// Set nodeList param for the Topic location.
//
// Set iVideo param for video index in the topics video array,
// or -1 if not a video search item.
//
// Used for locating a search item in the contents tree control.
BOOL CTopicTree::SearchGetItemPath(const CSearchItem & searchItem, CStringArray & nodeList,
								   int & iVideo)
{
	// Zero params.
	nodeList.RemoveAll();
	iVideo = -1;

	// Get num item nodes.
	int szNodes = searchItem.m_nodeIndices.GetSize();
	if(!szNodes)
		return FALSE;

	// Single node cannot be a video.
	if(szNodes == 1 && searchItem.m_isVideo)
		return FALSE;

	// Start at index 0 for the Root.
	int iNode = 0;

	// Get TopicTree index.
	int iTreeAr = searchItem.m_nodeIndices[iNode];

	// Get tree ar size - for roots.
	int szTreeAr = m_topics.GetSize();

	// Check bounds.
	if(iTreeAr >= szTreeAr)
		return FALSE;

	// Get the root Topic.
	CTopic & topic = m_topics[iTreeAr];

	// If this is the last node - not a video for roots.
	if(iNode + 1 == szNodes)
	{
		nodeList.Add(topic.treeTopic);
		return TRUE;
	}
	// Else add the topic node.
	else
	{
		nodeList.Add(topic.treeTopic);
	}

	// Set next node.
	int iNextNode = iNode + 1;

	// If this is the last node and a video.
	if(iNextNode + 1 == szNodes && searchItem.m_isVideo)
	{
		int iVidAr = searchItem.m_nodeIndices[iNextNode];
		int nVideos = topic.videos.GetSize();

		// Check bounds.
		if(iVidAr >= nVideos)
			return FALSE;

		// Set video index.
		iVideo = iVidAr;
		return TRUE;
	}

	// Else node is a child topic.
	else if(iNextNode < szNodes)
	{
		int nChildTopics = topic.childTopics.GetSize();

		// Get ar index.
		int arIndex = searchItem.m_nodeIndices[iNextNode];
		// Check bounds.
		if(arIndex >= nChildTopics)
			return FALSE;
		// Get a ref to the child topic.
		CTopic & childTopic = topic.childTopics[arIndex];
		// Call child.
		return childTopic.SearchTopicGetItemPath(searchItem, nodeList, iVideo, iNextNode);
	}
/**/
	// Problem - should not get here
	return FALSE;
}


// Read from archive.
BOOL CTopicTree::Read()
{
	int ret = TRUE; 

	// Get app data directory from CApp.
	CString curDir = ((CApp*)AfxGetApp())->GetAppDataFolder();
	curDir += _T("\\KhanDB.dat");

	// Call overload.
	return Read(curDir);
}

// Read from archive.
BOOL CTopicTree::Read(const CString & filePath)
{
	int ret = TRUE; 

	// Open the file for reading.
	CFile file;
	CFileException ex;
	if(!file.Open(filePath, CFile::modeRead | CFile::shareDenyWrite, &ex))
	{
	//	ex.ReportError();
		return FALSE;
	}
	if(file.GetLength() == 0)
	{
	//	msg.Format(_T("%s is an empty file"), filePathShort);
	//	AfxMessageBox(msg);
		TRACE(_T("ERROR CTopicTree Read GetLength() = 0\n"));
		return FALSE;
	}

	CArchive ar(&file, CArchive::load);
	TRY
	{	
		// Get schema
		int schema; // note GetObjectSchema() is for serializable classes
		ar >> schema;
		if(schema != 1)
		{
			AfxMessageBox(_T("KhanDB.dat is an invalid Data Base version."));
			
			// Close Archive
			ar.Close();
			// Close the file and ignore any assertions	
			file.Abort();
			return FALSE;
		}

		// Read
		int sz;
		ar >> sz;
		CTopic topic;
		for(int i = 0; i < sz; i++)
		{
			topic.Read(ar);
			m_topics.Add(topic);
			topic.Zero();
		}

		// Close the archive.	
		ar.Close();
		// Close the file	
		file.Close();
	}
	CATCH_ALL(e)
	{
		// Close the archive.	
		ar.Close();
		// Close file safely and quietly
		file.Abort(); 
		TRY
		{
			// CDoc member
			//ReportSaveLoadException(filePath, e, FALSE, AFX_IDP_FAILED_TO_OPEN_DOC);
		}
		END_TRY
		e->Delete();
		ret = FALSE;
	}
	END_CATCH_ALL

	return ret;
}
// Write to archive - seems ok
BOOL CTopicTree::Write()
{
	int ret = TRUE;

	// Get app data directory from CApp.
	CString curDir = ((CApp*)AfxGetApp())->GetAppDataFolder();
	curDir += _T("\\KhanDB.dat");	

	// Open the file for writing.
	CFile file;
	CFileException ex;
	if(!file.Open(curDir, CFile::modeCreate | CFile::modeWrite , &ex))
	{
		ex.ReportError();
		return FALSE;
	}

	CArchive ar(&file, CArchive::store);
	TRY
	{
		// Set schema
		int schema = 1; // note SetObjectSchema() is for serializable classes
		ar << 1;

		// Write.
		int sz = m_topics.GetSize();
		ar << sz;
		for(int i = 0; i < sz; i++)
		{
			m_topics[i].Write(ar);
		}

		// Close the archive.	
		ar.Close();
		// Close the file	
		file.Close();
	}
	CATCH_ALL(e)
	{
		// Close the archive.	
		ar.Close();
		// Close file safely and quietly
		file.Abort(); 
		TRY
		{
			// CDoc member
			//ReportSaveLoadException(curDir, e, TRUE, AFX_IDP_FAILED_TO_SAVE_DOC);
			// THROW_LAST();
		}
		END_TRY
		e->Delete();
		ret = FALSE;
	}
	END_CATCH_ALL

	return ret;
}

////////////////////////////////////////////////////////////////
// Class CVideoArray
//
// Sort Videos by date ascending (earliest first).
void CVideoArray::SortVideoDateAscend() 
{ 
	qsort(GetData(), GetSize(), sizeof(CVideo), CompareVideoDateAscend);
}

// Sort Videos by title ascending.
void CVideoArray::SortVideoTitlesAscend() 
{ 
	qsort(GetData(), GetSize(), sizeof(CVideo), CompareVideoTitlesAscend);
}

// Sort Videos by duration ascending.
void CVideoArray::SortVideoDurationAscend() 
{ 
	qsort(GetData(), GetSize(), sizeof(CVideo), CompareVideoDurationAscend);
}

// Sort Videos by title ascending, and remove any duplicates.
void CVideoArray::SortVideoTitlesAscendRemoveDuplicates()
{
	// Sort titles in alphabetical order.
	qsort(GetData(), GetSize(), sizeof(CVideo), CompareVideoTitlesAscend);

	// Loop in reverse deleting any duplicates.
	int sz = GetSize();
	for(int i = sz - 1; i > 0; i--)
	{
		CVideo & vid = ElementAt(i);
		CVideo & vidBefore = ElementAt(i - 1);

		if(vid.title == vidBefore.title)
		{
			/* dbg
			CString name1, name2;
			vid.GetShortFileName(name1);
			vidBefore.GetShortFileName(name2);*/

			// Delete dup.
			RemoveAt(i);
		}
	}
}

// Compare date added for two videos - ascending order.
int CVideoArray::CompareVideoDateAscend(const void *arg1, const void *arg2) 
{
	// Profile release: 0.6 %  19101 calls

	// Get the CVideos.
	CVideo * vid1 = (CVideo *)arg1;
	CVideo * vid2 = (CVideo *)arg2;
	int i, sz;
	CString buf;

	// Convert date/time strings to numeric: "2011-02-20T16:40:17Z" to 20110220164017
	//
	buf = vid1->date_added;
	CString int1;
	sz = buf.GetLength();
	for(i = 0; i < sz; i ++)
	{
		if(buf[i] > 47 && buf[i] < 58)
			int1 += buf[i];
	}

	buf = vid2->date_added;
	CString int2;
	sz = buf.GetLength();
	for(i = 0; i < sz; i ++)
	{
		if(buf[i] > 47 && buf[i] < 58)
			int2 += buf[i];
	}

	__int64 one = _wtoi64(int1);
	__int64 two = _wtoi64(int2);

	// Compare.
	//
	// Lesser.
	if(one < two)
		return -1;
	// Greater.
	else if(one > two)
		return +1;

	// Equal - todo -  test sort by tittle
	else
	{
	//	return vid1->title.CompareNoCase(vid2->title);
	}
	return 0;
}

// Compare titles for two videos.
int CVideoArray::CompareVideoTitlesAscend(const void *arg1, const void *arg2) 
{
	// Get the CVideos.
	CVideo * vid1 = (CVideo *)arg1;
	CVideo * vid2 = (CVideo *)arg2;
	return vid1->title.Compare(vid2->title);

	//return vid1->title.CompareNoCase(vid2->title);	// why?
}

// Compare duration for two videos.
int CVideoArray::CompareVideoDurationAscend(const void *arg1, const void *arg2) 
{
	// Get the CVideos.
	CVideo * vid1 = (CVideo *)arg1;
	CVideo * vid2 = (CVideo *)arg2;

	// Get duration as an UINT.
	UINT one = _wtoi(vid1->duration);
	UINT two = _wtoi(vid2->duration);

	// Compare.
	//
	// Lesser.
	if(one < two)
		return -1;
	// Greater.
	else if(one > two)
		return +1;

	// Equal.
	return 0;
}

////////////////////////////////////////////////////////////////
// Class CTopicArray
//
// Sort topics decending.
void CTopicArray::SortTopicsDecend() 
{ 
	qsort(GetData(), GetSize(), sizeof(CTopic), CompareTopicsDecend);
}

// Compare date added for two topics.
int CTopicArray::CompareTopicsDecend(const void *arg1, const void *arg2) 
{
	// Get the CTopics.
	CTopic * topic1 = (CTopic *)arg1;
	CTopic * topic2 = (CTopic *)arg2;

	// The Parents must match. 
	if (topic1->parentTreeTopic != topic2->parentTreeTopic)
		return 0;

	return topic1->title.CompareNoCase(topic2->title);

}

////////////////////////////////////////////////////////////////
// Class CSortStrArray
//
// Sort topics decending.
void CSortStrArray::SortStrDecend() 
{ 
	qsort(GetData(), GetSize(), sizeof(CString), CompareStrDecend);
}

// Compare date added for two topics.
int CSortStrArray::CompareStrDecend(const void *arg1, const void *arg2) 
{
	// Get the CTopics.
	CString str1 = *(CString *)arg1;
	CString str2 = *(CString *)arg2;
	return str1.Compare(str2);
}

////////////////////////////////////////////////////////////////
// Class CSortIntArray
//
// Sort topics decending.
void CSortIntArray::SortIntDecend() 
{ 
	qsort(GetData(), GetSize(), sizeof(int), CompareIntDecend);
}

// Compare ints for decending order.
int CSortIntArray::CompareIntDecend(const void *arg1, const void *arg2) 
{
	// Get the CTopics.
	int int1 = *(int *)arg1;
	int int2 = *(int *)arg2;

	if(int1 < int2)
		return 1;
	else if(int1 > int2)
		return -1;
	
	return 0;
}

//////////////////////////////////////////
// Tree node list array for params
// Fix 9/13/2017 - duplicate topic title fix
CNodeItem::CNodeItem()
{
	Zero();
}
CNodeItem::~CNodeItem()
{
}
CNodeItem::CNodeItem(const CNodeItem & nodeItem)
{
	m_title = nodeItem.m_title;
	m_arIndex = nodeItem.m_arIndex;
}
void CNodeItem::operator=(const CNodeItem & nodeItem)
{
	m_title = nodeItem.m_title;
	m_arIndex = nodeItem.m_arIndex;
}
BOOL CNodeItem::operator==(const CNodeItem & nodeItem)
{
	if(m_title == nodeItem.m_title && m_arIndex == nodeItem.m_arIndex)
		return TRUE;
	return FALSE;
}
BOOL CNodeItem::operator!=(const CNodeItem & nodeItem)
{
	return !(*this == nodeItem); 
}
void CNodeItem::Zero()
{
	m_title =_T("");
	m_arIndex = -1;
}
BOOL CNodeItem::IsValid() const
{
	if(!m_title.IsEmpty() && m_arIndex > -1)
		return TRUE;
	else
		return FALSE;
};

///////////////////////////////////////////////////////
// Class CSearch item - 10/04/2017 search

CSearchItem::CSearchItem()
{
	Zero();
}
CSearchItem::~CSearchItem()
{
}
CSearchItem::CSearchItem(const CSearchItem & searchItem)
{
	Copy(searchItem);
}

void CSearchItem::operator=(const CSearchItem & searchItem)
{
	Copy(searchItem);
}
BOOL CSearchItem::operator==(const CSearchItem & searchItem)
{
	return FALSE;
}

BOOL CSearchItem::operator!=(const CSearchItem & searchItem)
{
	return !(*this == searchItem); 
}
void CSearchItem::Copy(const CSearchItem & searchItem)
{
	m_nodeIndices.Copy(searchItem.m_nodeIndices);
	m_isVideo = searchItem.m_isVideo;

}
void CSearchItem::Zero()
{
	m_nodeIndices.RemoveAll();
	m_isVideo = FALSE;
}


//////////////////////////////////////////
// test - save demo code
// 
// sortable searchable CArray
int CSortSearchArray::CompareSort(const void *arg1, const void *arg2) 
{
	int int1 = *(int *)arg1;
	int int2 = *(int *)arg2;
	if (int1 < int2)
		return -1;
	else if (int1 > int2)
		return +1;
	return 0;
}

int CSortSearchArray::CompareSearch(const void *arg1, const void *arg2) 
{
	CString str1 = *(CString *)arg1;
	CString str2 = *(CString *)arg2;
	return str1.Compare(str2);
}

void CSortSearchArray::Sort() 
{
	qsort(GetData(), GetSize(), sizeof(int), CompareSort);
};


int * CSortSearchArray::Search(CString & title)
{
	// see bsearch - returns null if none!
	return (int *)bsearch(&title, GetData(), GetSize(), sizeof(int), CompareSearch);
};





