
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

#if !defined KHAN_DB_H
#define KHAN_DB_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Stdafx.h"
#include <afxtempl.h>

////////////////////////////////////////////////////////////////
// Sortable CArray for CVideos
class CVideo;
class CVideoArray : public CArray <CVideo, CVideo&> 
{
	static int CompareVideoDateAscend(const void *, const void *);
	static int CompareVideoTitlesAscend(const void *, const void *);
	static int CompareVideoDurationAscend(const void *, const void *);

	public:
	void SortVideoDateAscend();
	void SortVideoTitlesAscend();
	void SortVideoDurationAscend();
	void SortVideoTitlesAscendRemoveDuplicates();
};

////////////////////////////////////////////////////////////////
// Sortable CArray for CTopics
class CTopic;
class CTopicArray : public CArray <CTopic, CTopic&> 
{
	static int CompareTopicsDecend(const void *, const void *);

	public:
	void SortTopicsDecend();
};

////////////////////////////////////////////////////////////////
// Sortable CArray for CStrings
class CSortStrArray : public CArray <CString, CString&> 
{
	static int CompareStrDecend(const void *, const void *);

	public:
	void SortStrDecend();
};

////////////////////////////////////////////////////////////////
// Sortable CArray for ints
class CSortIntArray : public CArray <int, int&> 
{
	static int CompareIntDecend(const void *, const void *);

	public:
	void SortIntDecend();
};

///////////////////////////////////////////////////////////////
//test demo code - save
class CSortSearchArray : public CArray<CVideo, CVideo&>   
{
	static int CompareSort(const void *, const void *);
	static int CompareSearch(const void *, const void *);
	public:

	void Sort();
	int * Search(CString & title);
};

///////////////////////////////////////////////////////////////
// Fix 9/13/2017 - duplicate topic title fix
// Tree node list array for params.
class CNodeItem
{
public:
	CNodeItem();
	~CNodeItem();
	CNodeItem(const CNodeItem & nodeItem);
	void	operator=(const CNodeItem & nodeItem);
	BOOL	operator==(const CNodeItem & nodeItem);
	BOOL	operator!=(const CNodeItem & nodeItem);
	void	Zero();
	BOOL	IsValid() const; 
	CString m_title;
	int		m_arIndex;
};
class CNodeArray : public CArray<CNodeItem, CNodeItem&>   
{
};

// 10/04/2017 search
class CSearchItem
{
public:
	CSearchItem();
	~CSearchItem();
	CSearchItem(const CSearchItem & searchItem);

	void	operator=(const CSearchItem & searchItem);
	BOOL	operator==(const CSearchItem & searchItem);
	BOOL	operator!=(const CSearchItem & searchItem);
	void	Copy(const CSearchItem & searchItem);
	void	Zero();
	CWordArray m_nodeIndices;
	BOOL m_isVideo;
};
class CSearchArray : public CArray<CSearchItem, CSearchItem&> 
{
};


////////////////////////////////////////////////////////////////
// class CVideo
//
// 28 bytes empty.
// 3590 videos.
// 99 kb empty

class CVideo
{
public:
	CVideo();
	~CVideo();
	CVideo(const CVideo & video);

	void	operator=(const CVideo & video);
	BOOL	operator==(const CVideo & video);
	BOOL	operator!=(const CVideo & video);
	void	Copy(const CVideo & video);
	void	Zero();
	BOOL	GetShortFileNameTitleOld(CString & name);
	BOOL	GetShortFileName(CString & name);
	BOOL	CompareFileNamesTitleOld(const CString & fileNameTitle);
	BOOL	CompareFileNames(const CString & fileNameTitle);
	int		CompareMp4Strings(const CString & mp4Str);
	void	Read(CArchive & ar);
	void	Write(CArchive & ar);

	// test search
	// Get text
	BOOL GetText(CString & _description, CString & _title, UINT & nChars);
	BOOL GetText2(CString & buf, const CString & search, const int & searchLen, UINT & nFound);

	CString topic;
	CString date_added;
	CString description;
	CString mp4;
	CString duration;
	CString title;
};


////////////////////////////////////////////////////////////////
// class CTopic
//
// 52 bytes empty.
// 109 topics.
// 5.5 kb empty
// 
// CTopic is used to store a tree node and all its children.
//
// For example: math/algebra/absolute-value
//
// An array of CTopics can be created with:
//
//	CTopicArray TopicsAr;
//
// TopicsAr can now be used as an sortable, dynamically allocated tree.
//
class CTopic
{
public:
	CTopic();
	~CTopic();
	CTopic(const CTopic & topic);

	void	operator=(const CTopic & topic);
	BOOL	operator==(const CTopic & topic);
	BOOL	operator!=(const CTopic & topic);
	void	Copy(const CTopic & topic);
	void	Zero();
	int		HasChildTopic(const CString & childTreeTopic);
	BOOL	GetTopicRef(CTopic & topic, const CString & treeTopic);
	BOOL	GetChildTopicOld(CTopic & topic, CStringArray & nodeTitleList, int listElmt);
	BOOL	GetChildTopicByPos(CTopic & topic, CNodeArray & nodeAr, int nodeArElmt);
	BOOL	GetTopicByTreeTopic(CTopic & topic, const CStringArray & nodeList, int listElmt);
	BOOL	UpdateTopic(CTopic & newTopicData, CStringArray & nodeList, 
						int listElmt, BOOL stringsOnly);
	BOOL	GetTopicPath(const CString & treeTopicStr, CStringArray & nodeList);
	void	AddChildTopics(CTopic & topicToAdd, CStringArray & nodesList, 
						   int nodeElmtToAdd);
	BOOL	VideoExists(const CVideo & vid, const CStringArray & nodeList,
						int curNode);
	void	VideoInstances(const CString & vidTitle, CStringArray & topicsAr);
	void	VideoInstances(const CString & vidTitle, CVideoArray & videoAr);
	void	VideoInstancesMp4(const CVideo & vid, CVideoArray & videoAr);
	CString SetEmptyTopicTitle(const CString & treeTopicStr);
	void	Sort(BOOL & sortChild, BOOL & sortVideo);
	int		NumTopics();
	void	PrintTopic(CStdioFile & file, int nTabs = 0);
	BOOL	VideoFileUsedTitleOld(const CString & fileName, CVideo * pVideo = NULL);
	BOOL	VideoFileUsed(const CString & fileName, CVideo * pVideo = NULL);
	void	Read(CArchive & ar);
	void	Write(CArchive & ar);
	BOOL	DeleteTopic(const CStringArray & nodesList, int listElmt);
	BOOL	DeleteChildTopicByPos(const CNodeArray & nodeAr, int nodeArElmt); // 10/04/2017
	void	GetTopicVideos(int & nVideos, CVideoArray * videoAr = NULL);

	// 10/04/2017 search
	BOOL	SearchTopic(const CString & term, CSearchArray & results,
						CSearchItem & searchItem, int topicArIndex, 
						int & nSearched, int & nFound);
	BOOL	SearchTopicGetStrings(const CSearchItem & searchItem, CString & strTitle, 
								  CString & strDescription, int iCurNode);
	BOOL	SearchTopicGetNodeTitles(const CSearchItem & searchItem, 
									 CString & titlesStr, int iCurNode, int maxLen);
	BOOL	SearchTopicGetItemPath(const CSearchItem & searchItem, CStringArray & nodeList,
								   int & iVideo, int iCurNode);

	CString		treeTopic;
	CString		parentTreeTopic;
	CString		description;
	CString		title;
	CTopicArray childTopics;	// array - 20 b 
	CVideoArray videos;			// array - 20 b

	// Topic & Video arrays: 40 bytes of 52 total
	// Using prts to CArray instead:
	//  2 ptrs = 8 bytes + one array always allocated = 28. - 12 bytes wasted
	//
	// As of 2012:
	//  105 topics = 1260 Bytes, 1.2 KB wasted.
	//  Khan Library data saved via CArchive is 700 kb.
	//
	// As of 2015:
	//  2000 topics = 24000 Bytes, 23.4 kb wasted.
	//  Khan Library data saved via CArchive is 7887 kb.
};

////////////////////////////////////////////////////////////////
// class CTopicTree

class CTopicTree
{
public:
	CTopicTree();
	~CTopicTree();

	BOOL	AddTopic(CTopic & topicToAdd, CStringArray & nodesList);
	BOOL	GetTopicOld(CTopic & topic, CStringArray & nodeTitleList);
	BOOL	GetTopicByPos(CTopic & topic, CNodeArray & nodeAr);
	BOOL	GetTopicByTreeTopic(CTopic & topic, CStringArray & nodeList);
	BOOL	GetTopicPath(const CString & treeTopicStr, CStringArray & nodeList);
	BOOL	UpdateTopic(CTopic & newTopicData, CStringArray & nodeList, BOOL stringsOnly);
	BOOL	VideoExists(const CVideo & vid);
	BOOL	VideoInstances(const CString & vidTitle, CStringArray & topicsAr);
	BOOL	VideoInstances(const CString & vidTitle, CVideoArray & videoAr);
	BOOL	VideoInstancesMp4(const CVideo & vid, CVideoArray & videoAr);
	void	Copy(CTopicTree & tree);
	void	Sort(BOOL & sortRoot, BOOL & sortChild, BOOL & sortVideo);
	int		NumTopics();
	void	Zero();
	BOOL	VideoFileUsedTitleOld(const CString & fileName, CVideo * pVideo = NULL);
	BOOL	VideoFileUsed(const CString & fileName, CVideo * pVideo = NULL);
	void	PrintTree(CString filePath);
	int		GetTopicVideos(CTopic & topic, CVideoArray * videoAr = NULL);
	BOOL	Read();
	BOOL	Read(const CString & filePath);
	BOOL	Write();

	// 10/04/2017 search
	BOOL	Search(const CString & term, CSearchArray & results,
				   int & nSearched, int & nFound);
	BOOL	SearchGetStrings(const CSearchItem & searchItem, CString & strTitle, 
							 CString & strDescription);
	BOOL	SearchGetItemPath(const CSearchItem & searchItem, CStringArray & nodeList,
							  int & iVideo);
	BOOL	SearchGetNodeTitles(const CSearchItem & searchItem, CString & titlesStr, 
								int maxLen);
	friend	CString SearchFormatTitleStr(const CString & title, BOOL addSlash, int maxLen);

private:
	CString SetEmptyTopicTitle(const CString & treeTopicStr);
	BOOL	DeleteTopic(CStringArray & nodesList);
public:
	BOOL	DeleteTopicByPos(const CNodeArray & nodeAr); // 10/04/2017

public:
	// The Tree
	CTopicArray m_topics;
};
#endif //KHAN_DB_H
