
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// TabView.cpp : implementation file
//

#include "stdafx.h"
#include "Khan.h"
#include "TabView.h"

#include "TreeDialog.h"
#include "DownloadDialog.h"
#include "KhanDialog.h"
#include "SearchDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// From msdn Psview.exe 

// Related:
// msdn: Creating Sophisticated Tabbed Views with CTabView and the TabDemo App 
// msdn: SNAPVW


/////////////////////////////////////////////////////////////////////////////
// CTabView

// ID for the CTabCtrl
#define ID_TABCTRL  1111 

// Static for use by CSplitterView
int CTabView::m_leftPaneWidth = 0;

IMPLEMENT_DYNCREATE(CTabView, CView)

CTabView::CTabView()
{
	TRACE(_T("Constructing CTabView\n"));
	m_TabCtrl = NULL;
}

CTabView::~CTabView()
{
	TRACE(_T("Destructing CTabView\n"));

	// Clean CApp handle.
	((CApp *)AfxGetApp())->m_hwndTabView = NULL;

	// Delete the memory space for C++ tab control object.
	if (m_TabCtrl)
		delete m_TabCtrl;

	// Delete the memory space for the C++ dialog object.  The actual dialog
	// (or page) will be destroyed when tab control is destroyed because it
	// is the child window of the tab control (see code in CreatePages()).
	int num = m_DlgArray.GetSize();
	for (int i = 0; i < num; i++)
	{
		if (m_DlgArray[i])
			delete m_DlgArray[i];
	}
}


BEGIN_MESSAGE_MAP(CTabView, CView)
	//{{AFX_MSG_MAP(CTabView)
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_SETUP_FONT_TREE, OnSetupTreeFont)
	//}}AFX_MSG_MAP
	ON_NOTIFY(TCN_SELCHANGE, ID_TABCTRL, OnTabSelChange)
	ON_NOTIFY(TCN_SELCHANGING, ID_TABCTRL, OnTabSelChanging)
	ON_MESSAGE (WM_USER_PAGE_SWITCH, OnPageSwitch)


END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTabView drawing

void CTabView::OnDraw(CDC* pDC)
{
	TRACE(_T("in CTabView::OnDraw()\n"));
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here

	// Set Tab Font once to match dialog pages.
	if(!(HFONT)m_tabFont)
	{
		LOGFONT lf;
		m_DlgArray[0]->GetFont()->GetLogFont(&lf);
		lf.lfHeight += lf.lfHeight < 0 ? -1 : 1; 
		m_tabFont.CreateFontIndirect(&lf);
		m_TabCtrl->SetFont(&m_tabFont);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CTabView diagnostics

#ifdef _DEBUG
void CTabView::AssertValid() const
{
	CView::AssertValid();
}

void CTabView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTabView message handlers
void CTabView::OnSize(UINT nType, int cx, int cy) 
{
	TRACE(_T("in CTabView::OnSize()\n"));

	// Base class
	CView::OnSize(nType, cx, cy);

	// Reposition tab control to new size	
	m_TabCtrl->MoveWindow(0, 0, cx, cy);

	// Need top left margins and height of tab control
	RECT tabRect;
	m_TabCtrl->GetItemRect(0, &tabRect);

	// Need user tab selection index to get dialog
	int sel_index = m_TabCtrl->GetCurSel();

	// Size new dialog slightly smaller than tab control display area,
	// else new dialog will cover tab control's borders
	RECT leftPaneClient;
	GetClientRect(&leftPaneClient);
	RECT dlgRect;
	dlgRect.left	= leftPaneClient.left + 2;
	dlgRect.right	= leftPaneClient.right - 2;
	dlgRect.top		= tabRect.bottom + 2;
	dlgRect.bottom	= leftPaneClient.bottom - tabRect.bottom - 4;

	// Set dialog window pos
	m_DlgArray[sel_index]->SetWindowPos(NULL, dlgRect.left, dlgRect.top,
										dlgRect.right, dlgRect.bottom,
										SWP_NOZORDER | SWP_SHOWWINDOW);

	// Init static width for use by CSplitterView::OnSize()
	if(cx != 0)
		m_leftPaneWidth = cx;
}

int CTabView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	TRACE(_T("in CTabView::OnCreate()\n"));

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create a tab control which is a child window of the CTabView and
	// whose size will be determined in CTabView::OnSize().
	m_TabCtrl = new CTabCtrl;
	ASSERT(m_TabCtrl);

	if (m_TabCtrl->Create(WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS, 
						  CRect(0, 0, 0, 0), this, ID_TABCTRL) == FALSE)
		return -1;

	// Create all pages for the tab control.
	CreatePages();

	// Set CApp handle.
	((CApp *)AfxGetApp())->m_hwndTabView = m_hWnd;

	return 0;
}

void CTabView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
	// TODO: Add your specialized code here and/or call the base class

	TRACE(_T("in CTabView::OnInitialUpdate()\n"));
	CView::OnInitialUpdate();

	// Call to size and show property sheets (dialogs) 
	OnTabSelChange(NULL, NULL);		
}

// Show the newly selected page in the tab control, and size
// to tab control display area
void CTabView::OnTabSelChange(NMHDR* pnmhdr, LRESULT* pResult)
{
	TRACE(_T("in CTabView::OnTabSelChange()\n"));

	// Get index of selected tab.
	int index = m_TabCtrl->GetCurSel();

	// Display new tab. 
	// Note old tab has been hidden in OnTabSelChanging().
	TabSelChange(index);
	return;
	//*pResult = TRUE;	// no return value according to online documentation
}

// Show a new Tab.
// Called from: OnTabSelChange() and SwitchPages() only.
void CTabView::TabSelChange(int newTabIndex)
{
	TRACE(_T("in CTabView::TabSelChange()\n"));

	// Ensure one of three tabs (zero based).
	if(newTabIndex < 0 || newTabIndex > NUM_PAGES - 1 )	// 10/04/2017 search
		return;

	// Need top left margins and height of tab control
	RECT tabRect;
	m_TabCtrl->GetItemRect(0, &tabRect);

	// Size new dialog slightly smaller than tab control display area,
	// else new dialog will cover tab control's borders...
	RECT leftPaneClient;
	GetClientRect(&leftPaneClient);
	RECT dlgRect;
	dlgRect.left	= leftPaneClient.left + 2;
	dlgRect.right	= leftPaneClient.right - 2;
	dlgRect.top		= tabRect.bottom + 2;
	dlgRect.bottom	= leftPaneClient.bottom - tabRect.bottom - 4;

	// Set dialog window pos
	m_DlgArray[newTabIndex]->SetWindowPos(NULL, dlgRect.left, dlgRect.top,
										  dlgRect.right, dlgRect.bottom,
										  SWP_NOZORDER | SWP_SHOWWINDOW);

	// Set CDoc::m_rawStr CString member displayed by CDataView.
	//
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(pDoc)
	{
		CString str;
		
		// Contents tab.
		if(newTabIndex == 0)			
		{
			str =	_T("Contents Tab:\n\n")
					_T("\tSelect a Topic or Video to display information.\n\n")
					_T("\tTip:\n")
					_T("\t\tUse the arrow keys to quickly browse the contents.\n\n")
					_T("\tRight click on a Topic or Video")
					_T(" to display download and play options.\n\n")
					_T("\tPrefixes before a Video Title indicate:\n")
					_T("\t\t| the video has been viewed.\n")
					_T("\t\t the video has been downloaded to your computer.\n")
					_T("\t\t- the video has been added to the download list.\n");
			pDoc->m_rawStr = str;

			// Update for text only.
			pDoc->UpdateAllViews(NULL, RAW_STRING, NULL);

			// Init CDoc index var.
			pDoc->m_curTabViewIndex = PAGE_CONTENTS; // 10/04/2017
		}

		// Download tab.
		else if(newTabIndex == 1)
		{
			str =	_T("Download Tab:\n\n")
					_T("\tShows a list of videos you have selected for download,")
					_T(" and the storage location for all video files.\n")
					_T("\tPress the Change button to set a new location.\n\n")
					_T("\tPress the Download button to begin downloading.")
					_T(" The download may be canceled at any time.\n\n")
					_T("\tPress the Remove-All button to remove all videos")
					_T("  selected for download from the list.\n\n")
					_T("\tVideos will be downloaded in the background.");
			pDoc->m_rawStr = str;
			// Update for text and close list files if open.
			pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL); 
			// Init CDoc index var.
			pDoc->m_curTabViewIndex = PAGE_DOWNLOAD;	// 10/04/2017
		}

		// Update tab.
		else if(newTabIndex == 2)
		{
			str =	_T("Update Tab:\n\n")
					_T("\tSave the Contents to a Backup file - recommended")
					_T(" before updating.\n\n")
					_T("\tRestore the Contents from a backup file.\n\n")
					_T("\tUpdate the Contents with the most recent Khan Academy")
					_T(" topics and video playlists.")
					_T(" Set the topic and sub topic order by checking the appropriate box.")
					_T(" Note ordering of sub topics is not recommended (set unchecked).\n\n"); 
			pDoc->m_rawStr = str;
			// Update for text and close list files if open.
			pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL);
			// Init CDoc index var.
			pDoc->m_curTabViewIndex = PAGE_UPDATE;	// 10/04/2017
		}
		// Search tab.	// 10/04/2017 search
		else if(newTabIndex == 3)
		{
			// Get current search selection if available
			CString temp;
			CSearchDialog * pDlg = (CSearchDialog *)m_DlgArray[3];
			if(pDlg != NULL						&&
			   pDlg->GetSearchCurSelText(temp)	&&
			   !temp.IsEmpty()					)
			{
				str = temp;
			}
			// Else use intro text.
			else
			{
				str =	_T("Search Tab:\n\n")
						_T("\tSearch for a single word or phrase you want to find.")
						_T(" Enclose phrases in double quotes.\n\n")
						_T("\tExample searches:\n")
						_T("\t  Algebra\n")
						_T("\t  \"linear algebra\"\n")
						_T("\t  root\n")
						_T("\t  \"Square Root\"\n")
						_T("\t  \"Leonardo da Vinci\"\n\n")
						_T("\tSearches are not case-sensitive.\n\n") 
						_T("\tPress the Search button or press enter to begin the search.\n\n") 
						_T("\tResults are limited to the first 2000 items found.\n\n") 
						_T("\tOnly Topic and Video titles/descriptions are searched.")
						_T(" Video voice transcriptions are not available or searched.\n\n"); 
			}
			pDoc->m_rawStr = str;
			// Update for text and close list files if open.
			pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL);
			// Init CDoc index var.
			pDoc->m_curTabViewIndex = PAGE_SEARCH;
		}

	}
}

// Hide the currently visible page in the tab control.
void CTabView::OnTabSelChanging(NMHDR* pnmhdr, LRESULT* pResult)
{
	TRACE(_T("in CTabView::OnTabSelChanging()\n"));

	// Call GetCurSel() will return the index of the page
	// to be hidden.  This is NOT the newly selected page when 
	// OnTabSelChange() is called.
	int sel_index = m_TabCtrl->GetCurSel();

	// Hide the window.
	m_DlgArray[sel_index]->ShowWindow(SW_HIDE);
}

// Switch to a new Tab / Page per lParam.
// Called from: CDoc::OnToolsListFiles().
LRESULT CTabView::OnPageSwitch(WPARAM wParam, LPARAM lParam)	// WM_USER_PAGE_SWITCH
{
	int index;

	// Get new page index from lParam.
	if(lParam == PAGE_CONTENTS)			// CDoc def
		index = 0;
	else if(lParam == PAGE_DOWNLOAD)
		index = 1;
	else if(lParam == PAGE_UPDATE)
		index = 2;
	else if(lParam == PAGE_SEARCH)		// 10/04/2017 search
		index = 3;
	else
		return 0;

	// Get the index of the page to be hidden.
	int sel_index = m_TabCtrl->GetCurSel();

	// If no change, bail.
	if(sel_index == index)
		return 0;

	// Hide the currently showing page..
	m_DlgArray[sel_index]->ShowWindow(SW_HIDE);

	// Show new page with TabSelChange.
	TabSelChange(index);

	// Select the tab.
	m_TabCtrl->SetCurSel(index);

	return 0;
}

// Create the pages.
void CTabView::CreatePages()
{
	TRACE(_T("in CTabView::CreatePages()\n"));
	CDialog * dlg;

	// Create the Contents Tree page and add it to dialog array.
	dlg = new CTreeDialog;
	ASSERT(dlg);
	m_DlgArray.Add(dlg);
	VERIFY(dlg->Create(IDD_DIALOG_TREE, m_TabCtrl));

	// Save initial dlg rect to CApp member.
	dlg->GetClientRect(((CApp*)AfxGetApp())->m_treeDlgRect);

	// Create the Download page and add it to dialog array.
	dlg = new CDownloadDialog;
	ASSERT(dlg);
	m_DlgArray.Add(dlg);
	VERIFY(dlg->Create(IDD_DIALOG_DOWNLOAD, m_TabCtrl));
	dlg->GetClientRect(((CApp*)AfxGetApp())->m_downloadDlgRect);

	// Create the Khan page and add it to dialog array.
	dlg = new CKhanDialog;
	ASSERT(dlg);
	m_DlgArray.Add(dlg);
	VERIFY(dlg->Create(IDD_DIALOG_KHAN, m_TabCtrl));
	dlg->GetClientRect(((CApp*)AfxGetApp())->m_updateDlgRect);

	// Create the Search page and add it to dialog array.  10/04/2017 search
	dlg = new CSearchDialog;
	ASSERT(dlg);
	m_DlgArray.Add(dlg);
	VERIFY(dlg->Create(IDD_DIALOG_SEARCH, m_TabCtrl));
	dlg->GetClientRect(((CApp*)AfxGetApp())->m_searchDlgRect);

	// Get tab text from the dialog template.
	TCHAR str[50];
	TC_ITEM tci;
	tci.mask	= TCIF_TEXT;
	tci.iImage	= -1;
	for (int i = 0; i < NUM_PAGES; i++)
	{	
		// Get the caption from dialog template and insert this caption
		// to the tab control.
		dlg = m_DlgArray[i];
		dlg->GetWindowText(str, sizeof(str) / sizeof(TCHAR));
		tci.pszText	= str;
		m_TabCtrl->InsertItem(i, &tci);

		// Remove caption from the dialog box because no caption is allowed
		// for pages in the tab control.
		dlg->ModifyStyle(WS_CAPTION, 0);

		// IMPORTANT: Why sending a WM_NCACTIVATE message to the dialog box?
		// When writing this sample, we found a problem which is related to
		// the edit control refuses to give up the input focus when mouse is 
		// being clicked on some other edit control.  This problem only 
		// happens when title bar is found from a dialog resource.  And we
		// need the title bar because it is the text in the tab control.  
		// Sending a WM_NCACTIVATE message seems to fix the problem.
		dlg->SendMessage(WM_NCACTIVATE, TRUE);
	}
}


void CTabView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default

	int iSel = m_TabCtrl->GetCurSel();
	CRect r;
	m_TabCtrl->GetItemRect(iSel, &r);
	ScreenToClient(&point);
	if(r.PtInRect(point))
	{
		int g = 0;
	}	
	CView::OnLButtonDown(nFlags, point);
}

// Process the ID_SETUP_FONT_TREE message.
// Not used ?
void CTabView::OnSetupTreeFont() 
{
	// Forward to tree.
	HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
	if(hWnd)
	{
		const MSG* msg = GetCurrentMessage();
		::PostMessage(hWnd, msg->message, msg->wParam, msg->lParam);
	}	
}
