
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// TreeDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Khan.h"
#include "TreeDialog.h"
#include "DataView.h"
#include "FontSetup.h"
#include "math.h"

//test dump
#include "Dump.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CTreeDialog dialog

CTreeDialog::CTreeDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CTreeDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTreeDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	TRACE(_T("constructing CTreeDialog\n"));

	// Flag used by OnSetSelLastViewed().
	m_manualExpand = FALSE;
}


void CTreeDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTreeDialog)
	DDX_Control(pDX, IDC_KHAN_TREE, m_treeCtrl);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTreeDialog, CDialog)
	//{{AFX_MSG_MAP(CTreeDialog)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_NOTIFY(TVN_ITEMEXPANDING, IDC_KHAN_TREE, OnItemexpandingKhanTree)
	ON_NOTIFY(TVN_SELCHANGED, IDC_KHAN_TREE, OnSelchangedKhanTree)
	ON_NOTIFY(TVN_SELCHANGING, IDC_KHAN_TREE, OnSelchanging)
	ON_COMMAND(ID_SETUP_FONT_TREE, OnSetupTreeFont)
	//}}AFX_MSG_MAP

	ON_NOTIFY(TVN_GETINFOTIP, IDC_KHAN_TREE, OnTvnGetInfoTip)
	ON_MESSAGE (WM_USER_DLG_UPDATE, OnUpdateTree)
	ON_MESSAGE (WM_USER_TREE_VID_UPDATE, OnUpdateVideoMarks)
	ON_MESSAGE (WM_USER_TREE_SET_SEL, OnSetSelLastViewed)
	ON_MESSAGE (WM_USER_TREE_SET_SEARCH, OnSetSelSearch) // 10/04/2017 search
	ON_MESSAGE (WM_USER_TREE_DELETE_SEL, OnDeleteSel)	 // 10/04/2017 developer utility

	ON_WM_CONTEXTMENU()

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeDialog message handlers

void CTreeDialog::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	TRACE(_T("in CTreeDialog::OnSize()\n"));

	// Create a rect with margins for tree control sizing.
	RECT treeRect;
	treeRect.top = 5;
	treeRect.left = 5;
	treeRect.bottom = cy - 5;
	treeRect.right = cx - 5;

	// Size the control.
	CWnd* pWnd = GetDlgItem(IDC_KHAN_TREE);
	if(pWnd != NULL)
		pWnd->MoveWindow(&treeRect);	
}

BOOL CTreeDialog::OnInitDialog() 
{
	TRACE(_T("in CTreeDialog::OnInitDialog()\n"));

	// Set CApp handle.
	((CApp *)AfxGetApp())->m_hwndTreeDlg = m_hWnd;

	CTreeCtrl * pTreeCtrl;
	pTreeCtrl = (CTreeCtrl*) GetDlgItem(IDC_KHAN_TREE);
	if(pTreeCtrl)
	{
		CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;

		// Set Tree Controls font.
		if(pDoc)
		{
			LOGFONT lf = pDoc->m_treeLf;
			m_treeFont.CreateFontIndirect(&lf);
			pTreeCtrl->SetFont(&m_treeFont);
		}

		// Set ToolTip and InfoTip duration.
		CToolTipCtrl * pToolTip = pTreeCtrl->GetToolTips();
		if(pToolTip)
		{
			pToolTip->SetDelayTime(TTDT_INITIAL, 500);		// Time to display - 500 default
			pToolTip->SetDelayTime(TTDT_RESHOW, 500);		// Time to reshow
			pToolTip->SetDelayTime(TTDT_AUTOPOP,  10000);	// Time displayed  -  ?  default
			pToolTip->SetTipBkColor(RGB(255, 251, 240) );	// win cream
			CRect rect(4,4,4,4);
			pToolTip->SetMargin(&rect);						// 4 pix dist to text
		}
	}
	return CDialog::OnInitDialog();
}

// Called from CDoc::OnNewDocument() during start up 
// and after a Khan Update.
LRESULT CTreeDialog::OnUpdateTree(WPARAM wParam, LPARAM lParam)	// WM_USER_DLG_UPDATE
{
	TRACE(_T("in CTreeDialog::OnUpdateTree()\n"));

	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(pDoc)
	{
		CTreeCtrl * pTreeCtrl;
		pTreeCtrl = (CTreeCtrl*) GetDlgItem(IDC_KHAN_TREE);

		// Delete existing.
		int nItems = pTreeCtrl->GetCount();
		if(nItems != 0)
		{
			TRACE(_T("In CTreeDialog OnUpdateTree - deleting all %d items.\n"), nItems);
			pTreeCtrl->DeleteAllItems();
		}

		HTREEITEM item;

		// Add the root items.
		CTopicTree & tree = pDoc->GetTree();
		int sz = tree.m_topics.GetSize();
		CString str;
		for(int i = 0; i < sz; i++)
		{
			str = tree.m_topics[i].title;
			item = pTreeCtrl->InsertItem(str, TVI_ROOT, TVI_LAST);

			// Fix 9/13/2017 - duplicate topic title fix
			// Save element this item is stored under.
			// Element is only valid for this tree indent. 
			pTreeCtrl->SetItemData(item, (DWORD) i);

			if(tree.m_topics[i].childTopics.GetSize() ||
			   tree.m_topics[i].videos.GetSize())
			{
				// Insert a dummy item - will be populated 
				// in OnItemexpandingKhanTree().
				pTreeCtrl->InsertItem(DUMMY_ITEM, item, item);
			}
		}
		return TRUE;
	}
	return FALSE;
}

/////////////////////////////////////////////////////////////////////
// Video prefix (mark) updating.

// Called from .
//
// Update any visable video prefixes (marks) to reflect 
// current marked-for-download or on-disk or Viewed status.
LRESULT CTreeDialog::OnUpdateVideoMarks(WPARAM wParam, LPARAM lParam) // WM_USER_TREE_VID_UPDATE
{
	CString str;
	BOOL ret = FALSE;

	// Loop the root items (7 or so).
	HTREEITEM hItem = m_treeCtrl.GetRootItem();
	while(hItem)
	{
		// dbg.
		//str = m_treeCtrl.GetItemText(hItem);

		// Process any item children.
		if(m_treeCtrl.ItemHasChildren(hItem))
			ret = UpdateVideoMarksChild(hItem);

		// Get next root.
		hItem = m_treeCtrl.GetNextSiblingItem(hItem);
	}
	// Imediately redraw clip set by UpdateVideoMarksChild().
	// Note clip rect set in called functions.
	if(ret)
		m_treeCtrl.UpdateWindow();

	return 0;
}

// Update child Video Mark (prefix).
BOOL CTreeDialog::UpdateVideoMarksChild(HTREEITEM hItem)
{
	CString str;
	BOOL ret = FALSE;

	// Make sure
	if(!hItem)
		ret;

	// Get the first subitem.
	HTREEITEM hNext = m_treeCtrl.GetChildItem(hItem);
	if(hNext)
	{
		// Process.
		if(SetVideoMarks(hNext))
			return TRUE;
	}
	else
		return ret;

	// If the subitem has children, make recursive call.
	if(m_treeCtrl.ItemHasChildren(hNext))
		UpdateVideoMarksChild(hNext);

	// If sibling subitems of hItem.
	while((hNext = m_treeCtrl.GetNextSiblingItem(hNext)) != NULL) 
	{
		// Process this sibling.
		if(SetVideoMarks(hNext))
			return TRUE;

		// If the sibling has children, make recursive call.
		if(m_treeCtrl.ItemHasChildren (hNext))
			ret = UpdateVideoMarksChild(hNext);
	}
	return ret;
}

// Set a Video item mark (prefix) by adding
// its rect to the update region. Prefixes
// are drawn via custon draw in OnNotify().
BOOL CTreeDialog::SetVideoMarks(HTREEITEM hItem)
{
	CRect invalid, rect;
	invalid.SetRectEmpty();

	// Get items text.
	CString str = m_treeCtrl.GetItemText(hItem);

	// If a dummy item.
	if(str == DUMMY_ITEM)
		return FALSE;

	// If this is a Video item, it means the Topic
	// is expanded and all vids are showing, and 
	// this is the first video in the topic.
	//
	// We will check each video in this topic, and
	// return TRUE if it needs redrawing.
	//
	// The caller, UpdateVideoMarksChild(), will then
	// move to the next topic.
	CString s = str.Right(VID_POSTFIX_LEN);
	if(s == VID_POSTFIX)
	{
		// Get all parents of this video item.
		//
		// Get vids Parent Topic handle.
		HTREEITEM hParent = m_treeCtrl.GetParentItem(hItem);

		// Fix 9/13/2017 - duplicate topic title fix
		// Get this items parent topic tree nodes and their ar indices.
		CNodeArray nodeAr;
		if(!hParent || !GetPathNodesFromItem(hParent, nodeAr))
			return FALSE;

		// Need a CDoc ptr.
		CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
		if(!pDoc)
			return FALSE;

		// Get the Parent Topic.
		CTopicTree & tree = pDoc->GetTree();
		CTopic topic;

		// Fix 9/13/2017 - duplicate topic title fix
		// Get the expanding topic.
		if(tree.GetTopicByPos(topic, nodeAr))
		{
			int sz = topic.videos.GetSize();
			BOOL viewed;

			// hItem is the first vid.
			HTREEITEM hVidItem = hItem;

			for(int i = 0; i < sz; i++)
			{
				CVideo & vid = topic.videos[i];
				viewed = pDoc->m_history.BeenViewed(vid);

				// If on disk.
				if(pDoc->VideoOnDisk(vid) && hVidItem)
				{
					// Ensure child item matches ar element.
					s = m_treeCtrl.GetItemText(hVidItem);
					if(s.Find(vid.title) != -1)
					{
						m_treeCtrl.GetItemRect(hVidItem, &rect, FALSE);
						invalid.UnionRect(invalid, rect);
					}
				}

				// If marked for download.
				else if(pDoc->VideoForDownload(vid))
				{
					// Ensure child item matches ar element.
					s = m_treeCtrl.GetItemText(hVidItem);
					if(s.Find(vid.title) != -1)
					{
						m_treeCtrl.GetItemRect(hVidItem, &rect, FALSE);
						invalid.UnionRect(invalid, rect);
					}
				}

				// Else the item is for streaming.
				else if(hVidItem)
				{
					// Ensure child item matches ar element.
					s = m_treeCtrl.GetItemText(hVidItem);
					if(s.Find(vid.title) != -1)
					{
						m_treeCtrl.GetItemRect(hVidItem, &rect, FALSE);
						invalid.UnionRect(invalid, rect);
					}
				}
				// Next child.
				if(hVidItem)
					hVidItem = m_treeCtrl.GetNextItem(hVidItem, TVGN_NEXT);
			}
			// Invalidate as needed. Note caller must 
			// call UpdateWindow().
			if(!invalid.IsRectEmpty())
			{
				m_treeCtrl.InvalidateRect(&invalid);
				return TRUE;
			}
		}
	}
	// Not a video item.
	return FALSE;
	
}

// Display and select last viewed Video. 
LRESULT CTreeDialog::OnSetSelLastViewed(WPARAM wParam, LPARAM lParam) // WM_USER_TREE_SET_SEL
{
	TRACE(_T("Entering CTreeDialog OnSetSelLastViewed\n"));

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return 0;

	// Not if playing.
	if(pDoc->m_videoPlaying)
		return 0;

	// Get the CDoc::m_historyLastViewed video.
	CVideo curVideo = pDoc->m_historyLastViewed;

	// Ensure.
	if(curVideo.topic.IsEmpty())
		return 0;

	// Get topicTree ref.
	CTopicTree & tree = pDoc->GetTree();

	// Get the path to the topic containing the video.
	CStringArray pathAr;
	if(!tree.GetTopicPath(curVideo.topic, pathAr))
	{
		TRACE(_T("ERROR CTreeDialog OnSetSelLastViewed GetTopicPath failed\n"));
		return FALSE;
	}
	int sz, i;
	CTopic topic;
	CString itemTitleStr, topicTitleStr, treeTopicStr;

	// Topic searched for must be last.
	sz = pathAr.GetSize();
	if(pathAr[sz - 1] != curVideo.topic)
	{
		TRACE(_T("ERROR CTreeDialog OnSetSelLastViewed pathAr wrong\n"));
		return 0;
	}

	/* dbg
	CString s;
	for(i = 0; i < sz; i++) 
	{ s += pathAr[i]; s += _T("\\"); }*/

	// Set flag used in OnItemexpandingKhanTree().
	m_manualExpand = TRUE;

	// We're getting the path's root.
	int pathArIndex = 0;

	// Num root topics.
	sz = tree.m_topics.GetSize();

	// Get the first treeCtrl root tree item.
	HTREEITEM hRoot = m_treeCtrl.GetNextItem(NULL, TVGN_ROOT);
	HTREEITEM hChild = NULL;

	// Collapse / clean all treeCtrl root items.
	while(hRoot)
	{
		DeleteAllChildren(hRoot);
		m_treeCtrl.InsertItem(DUMMY_ITEM, hRoot);	// add the dummy
		m_treeCtrl.Expand(hRoot, TVE_COLLAPSE);		// collapse the dummy, show the +
		// Next
		hRoot = m_treeCtrl.GetNextItem(hRoot, TVGN_NEXT);
	}

	// Loop root topics.
	hRoot = m_treeCtrl.GetNextItem(NULL, TVGN_ROOT);
	for(i = 0; i < sz; i++)
	{
		// Ensure valid treeCtrl item.
		if(!hRoot)
		{
			TRACE(_T("ERROR CTreeDialog OnSetSelLastViewed hRoot NULL\n"));
			break;
		}

		// Get the topicTree topic ref.
		CTopic & topic = tree.m_topics[i];

		// Get topicTree topic title.
		topicTitleStr = topic.title;

		// Get topicTree topic str.
		treeTopicStr = topic.treeTopic;

		// Get coresponding treeCtrl item title.
		itemTitleStr = m_treeCtrl.GetItemText(hRoot);

		// Ensure title's match.
		if(itemTitleStr != topicTitleStr)
		{
			TRACE(_T("ERROR CTreeDialog OnSetSelLastViewed itemTitleStr != topicTitleStr\n"));
			break;
		}

		// If at the video's topicTree root topic.
		if(treeTopicStr == pathAr[pathArIndex])
		{
			// Remove the dummy item added above.
			if(m_treeCtrl.ItemHasChildren(hRoot))
			{
			   HTREEITEM hDummy = m_treeCtrl.GetChildItem(hRoot);
			   if(hDummy && m_treeCtrl.GetItemText(hDummy) == DUMMY_ITEM)
				   m_treeCtrl.DeleteItem(hDummy);
			}

			// Expand root's child topics.
			ExpandChildTopic(topic, curVideo, pathAr, hRoot, pathArIndex);

			// Done.
			break;
		}
		// Get next treeCtrl root.
		hRoot = m_treeCtrl.GetNextItem(hRoot, TVGN_NEXT);
	}

	// Clear flag.
	m_manualExpand = FALSE;

	// Set focus to Tree.
	m_treeCtrl.SetFocus();

	return 0;
}

// 10/04/2017 search
// Display and select selected search item. 
LRESULT CTreeDialog::OnSetSelSearch(WPARAM wParam, LPARAM lParam) // WM_USER_TREE_SET_SEARCH
{
	TRACE(_T("Entering CTreeDialog OnSetSelSearch\n"));

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return 0;

	// Not if playing.
	if(pDoc->m_videoPlaying)
		return 0;

	// Get the node path and videoIndex of search item.
	CStringArray & nodeAr = pDoc->m_searchNodes;
	int iVideo = pDoc->m_searchVideoIndex;

	// Must have a node.
	int nNodes = nodeAr.GetSize();
	if(!nNodes)
		return 0;

	/* dbg
	CString s;
	for(int ii = 0; ii < nNodes; ii++) 
	{ s += nodeAr[ii]; s += _T("\\"); }*/


	// Get topicTree ref.
	CTopicTree & tree = pDoc->GetTree();

	int sz, i;
	CTopic topic;
	CString itemTitleStr, topicTitleStr, treeTopicStr;

	// Set flag used in OnItemexpandingKhanTree().
	m_manualExpand = TRUE;

	// First node is for the root topic.
	int iRootNode = 0;

	// Num root topics.
	sz = tree.m_topics.GetSize();

	// Get the first treeCtrl root tree item.
	HTREEITEM hRoot = m_treeCtrl.GetNextItem(NULL, TVGN_ROOT);
	HTREEITEM hChild = NULL;

	// Collapse, clean and add dummy items to all 
	// treeCtrl root items.
	while(hRoot)
	{
		DeleteAllChildren(hRoot);
		m_treeCtrl.InsertItem(DUMMY_ITEM, hRoot);	// add the dummy
		m_treeCtrl.Expand(hRoot, TVE_COLLAPSE);		// collapse the dummy, show the +
		// Next
		hRoot = m_treeCtrl.GetNextItem(hRoot, TVGN_NEXT);
	}

	// Traverse root topics to the search root..
	hRoot = m_treeCtrl.GetNextItem(NULL, TVGN_ROOT);
	for(i = 0; i < sz; i++)
	{
		// Ensure valid treeCtrl item.
		if(!hRoot)
		{
			TRACE(_T("ERROR CTreeDialog OnSetSelSearch hRoot NULL\n"));
			break;
		}
		// Get a ref to the topicTree topic.
		CTopic & topic = tree.m_topics[i];

		// If at the nodeAr[0] root Topic index.
		if(topic.treeTopic == nodeAr[iRootNode])
		{
			// Get idem data - matches indices.
			int iData = m_treeCtrl.GetItemData(hRoot);

			// Item data must match index.
			if(iData != i)
			{
				TRACE(_T("ERROR CTreeDialog OnSetSelSearch index bounds error.\n"));
				break;
			}
			// Remove the dummy item added above.
			if(m_treeCtrl.ItemHasChildren(hRoot))
			{
			   HTREEITEM hDummy = m_treeCtrl.GetChildItem(hRoot);
			   if(hDummy && m_treeCtrl.GetItemText(hDummy) == DUMMY_ITEM)
				   m_treeCtrl.DeleteItem(hDummy);
			}
			// Expand this root's child topics.
			if(!ExpandChildTopicSearch(topic, hRoot, nodeAr, iVideo, iRootNode))
			{
				TRACE(_T("ERROR CTreeDialog OnSetSelSearch expand child error.\n"));
			}
			// Done.
			break;
		}
		// Else get next treeCtrl root.
		else
			hRoot = m_treeCtrl.GetNextItem(hRoot, TVGN_NEXT);
	}
	// Clear flag.
	m_manualExpand = FALSE;

	// Set focus to Tree.
	m_treeCtrl.SetFocus();

	return 0;
}

// Delete the current selection - developer utility only 
// called from View's OnTest().
LRESULT CTreeDialog::OnDeleteSel(WPARAM wParam, LPARAM lParam) // WM_USER_TREE_DELETE_SEL
{
	TRACE(_T("Entering CTreeDialog OnDeleteSel\n"));

	// Get the current selection.
	HTREEITEM hSelItem = m_treeCtrl.GetSelectedItem();
	if(hSelItem == NULL)
		return 0;

	// Ensure sel item is a topic.
	//
	CString str = m_treeCtrl.GetItemText(hSelItem);
	// If a dummy item.
	if(str == DUMMY_ITEM)
		return FALSE;
	CString s = str.Right(VID_POSTFIX_LEN);
	if(s == VID_POSTFIX)
		return 0;

	// Get the item path as topic titles.
	CNodeArray nodeAr;
	if(!GetPathNodesFromItem(hSelItem, nodeAr))
		return 0;

	// Ensure.
	int nNodes = nodeAr.GetSize();
	if(!nNodes)
		return 0;

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return 0;

	// Ensure not playing.
	if(pDoc->m_videoPlaying)
		return 0;

	// Should match. Could have just used this pDoc hItem.
	if(pDoc->m_hCurTopic != hSelItem)
		return 0;

	// Get topicTree ref.
	CTopicTree & tree = pDoc->GetTree();

	// Delete.
	if(tree.DeleteTopicByPos(nodeAr))
	{
		TRACE(_T("CTreeDialog OnDeleteSel - sel deleted ok.\n"));

		// Update the tree.
		OnUpdateTree(0, 0);

		// Save the new tree. Note modified TopicTree is in memory only.
		// Once written, a backup can be saved.
		tree.Write();
	}

	return 0;
}



// Create and Expand a treeCtr topic.
//
// Note topic has been collapsed by OnSetSelLastViewed(), and 
// will completely redraw including video prefixes.
//
// Called only from OnSetSelLastViewed() above.
// Uses recursion.
BOOL CTreeDialog::ExpandChildTopic(CTopic & parentTopic, CVideo & curVideo,  
								   CStringArray & pathAr, HTREEITEM hParentItem,
								   int parentPathArIndex)
{
	TRACE(_T("Entering CTreeDialog ExpandChildTopic\n"));

	// Note: Assuming all root tree items (7 or so) have 
	// been collapsed and cleaned.

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		TRACE(_T("ERROR CTreeDialog ExpandChildTopic pDoc NULL\n"));
		return FALSE;
	}

	// Ensure valid index.
	int pathArSz = pathAr.GetSize();
	if(parentPathArIndex >= pathArSz)
	{
		TRACE(_T("ERROR CTreeDialog ExpandChildTopic invalic pathAr index\n"));
		return FALSE;
	}
	/* dbg
	CString s;
	for(int i = 0; i < pathArSz; i++) 
	{ s += pathAr[i]; s += _T("\\"); }*/


	CString topicTitleStr, treeTopicStr;
	int nTopics = parentTopic.childTopics.GetSize();
	int nVideos = parentTopic.videos.GetSize();
	int iTopic, iVideo;
	HTREEITEM hNewItem;

	// Add all child topics to tree.
	for(iTopic = 0; iTopic < nTopics; iTopic++)
	{
		// Get child Topic ref.
		CTopic & childTopic =  parentTopic.childTopics[iTopic];

		// Get topicTree title str.
		topicTitleStr = childTopic.title;

		// Get topicTree topic str.
		treeTopicStr = childTopic.treeTopic;

		// Insert child below parent.
		TRACE(_T("Inserting child %d : %s\n"), iTopic, topicTitleStr);
		hNewItem = m_treeCtrl.InsertItem(topicTitleStr, hParentItem, TVI_LAST);
		// MFC calls:
		// Note TVINSERTSTRUCT tvis - item.mask == 1 (TVIF_TEXT)
		// SendMessage(m_hWnd, TVM_INSERTITEM, 0, (LPARAM)&tvis)

		// Ensure.
		if(!hNewItem)
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopic hNewItem NULL\n"));
			return FALSE;
		}

		// Fix 9/13/2017 - duplicate topic title fix
		// Save element this item is stored under.
		// Element is only valid for this tree indent. 
		m_treeCtrl.SetItemData(hNewItem, (DWORD) iTopic);

		// If at the next child treeTopic on the path,
		// make a recursive call.
		if(parentPathArIndex + 1 <= pathArSz			 &&
		   treeTopicStr == pathAr[parentPathArIndex + 1] )
		{
			TRACE(_T("CTreeDialog ExpandChildTopic - recursive call\n"));

			// Expand this child topic.
			if(!ExpandChildTopic(childTopic, curVideo, pathAr, 
								 hNewItem, parentPathArIndex + 1))
			{
				return FALSE;
			}
			TRACE(_T("CTreeDialog ExpandChildTopic - post recursive call\n"));
		}

		// Else add dummy item below child.
		else
		{
			// If Topic has videos or child topics, add a dummy child.
			if(childTopic.childTopics.GetSize() || 
			   childTopic.videos.GetSize()		)
			{
				TRACE(_T("Inserting and collapsing DUMMY_ITEM\n"));
				m_treeCtrl.InsertItem(DUMMY_ITEM, hNewItem, hNewItem);
				m_treeCtrl.Expand(hNewItem, TVE_COLLAPSE);	// collapse the dummy, show the +
			}

			// Some Topics are empty.
			else
			{
				TRACE(_T("Inserting and collapsing EMPTY_TOPIC\n"));
				m_treeCtrl.InsertItem(EMPTY_TOPIC, hNewItem, hNewItem);
				m_treeCtrl.Expand(hNewItem, TVE_COLLAPSE);	// collapse the dummy, show the +
			}
		}
	}

	// Add all videos to the tree.
	// If at the target video, set as current selection.
	// Note a Topic may have vids and sub-topics.
	CString str;
	HTREEITEM hVideoItem = NULL;
	HTREEITEM hSelVideoItem = NULL;
	for(iVideo = 0; iVideo < nVideos; iVideo++)
	{
		CVideo & video = parentTopic.videos[iVideo];

		// Get the Title.
		str = parentTopic.videos[iVideo].title;

		// Add the postfix marker
		str += VID_POSTFIX;

		// Insert hItem video.
		hVideoItem = m_treeCtrl.InsertItem(str, hParentItem, TVI_LAST);

		// Ensure.
		if(!hVideoItem)
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopic hVideoItem NULL\n"));
			return FALSE;
		}

		// If the video we are after.
		if(curVideo.title == video.title)
		{
			// Set pDoc members.
			pDoc->m_curVideo = video;
			pDoc->m_hCurVideo = hVideoItem;
			
			// Set string array for view display.
			pDoc->SetDisplayStringsVideo();

			// Save handle.
			hSelVideoItem = hVideoItem;
		}
	}
	// Done
	//
	// Highlite treeCtrl Video item if inserted.
	if(hSelVideoItem)
		m_treeCtrl.SelectItem(hSelVideoItem);

	// Ok
	return TRUE;
}

// 10/04/2017 search
// Create and Expand a searched for treeCtr topic/videp.
//
// Note topic has been collapsed by OnSetSelSearch(), and 
// will completely redraw including video prefixes.
//
// Called only from OnSetSelSearch() above.
// Uses recursion.
BOOL CTreeDialog::ExpandChildTopicSearch(CTopic & parentTopic, HTREEITEM hParentItem, 
										 const CStringArray & nodeAr, const int & nodeVideoIndex,
										 int iNodeArParent)
{
	TRACE(_T("Entering CTreeDialog ExpandChildTopicSearch\n"));

	// Note: Assuming all root tree items (7 or so) have 
	// been collapsed and cleaned.

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch pDoc NULL\n"));
		return FALSE;
	}

	// Ensure valid index.
	int nodeArSz = nodeAr.GetSize();
	if(iNodeArParent >= nodeArSz)
	{
		TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch invalid nodeAr index\n"));
		return FALSE;
	}

	/* dbg*/
	CString s;
	for(int i = 0; i < nodeArSz; i++) 
	{ s += nodeAr[i]; s += _T("\n"); }

	// Get counts.
	int nChildTopics = parentTopic.childTopics.GetSize();
	int nVideos = parentTopic.videos.GetSize();

	//////////////////////////////////////////////
	// If displaying this topics text - no video.
	if(iNodeArParent == nodeArSz -1 && nodeVideoIndex == -1)
	{
		// Ensure a dummy item was not added here.
		HTREEITEM hDummy = m_treeCtrl.GetChildItem(hParentItem);
		if(hDummy && m_treeCtrl.GetItemText(hDummy) == DUMMY_ITEM)
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch dummy item error\n"));
			return FALSE;
		}
		// If Topic has videos or child topics, add a Dummy Topic child item.
		if(parentTopic.childTopics.GetSize() || 
		   parentTopic.videos.GetSize()		)
		{
			TRACE(_T("Inserting and collapsing DUMMY_ITEM\n"));
			m_treeCtrl.InsertItem(DUMMY_ITEM, hParentItem, hParentItem);
			m_treeCtrl.Expand(hParentItem, TVE_COLLAPSE);	// collapse the dummy, show the +
		}
		// Else add a Empty Topic child item - some Topics are empty.
		else
		{
			TRACE(_T("Inserting and collapsing EMPTY_TOPIC\n"));
			m_treeCtrl.InsertItem(EMPTY_TOPIC, hParentItem, hParentItem);
			m_treeCtrl.Expand(hParentItem, TVE_COLLAPSE);	// collapse the dummy, show the +
		}
		// Set CDoc and select item
		pDoc->m_curTopic =  parentTopic;
		pDoc->m_hCurTopic = hParentItem;
			
		// Set string array for view display.
		pDoc->SetDisplayStringsTopic();

		// Highlight.
		m_treeCtrl.SelectItem(hParentItem);

		// Done
		return TRUE;
	}

	///////////////////////////////////////////////
	// Else if displaying text for a video in this 
	// topic and the topic has no child topics.
	else if(iNodeArParent == nodeArSz - 1 && nodeVideoIndex != -1 &&
			nChildTopics == 0 && nVideos)
	{
		// Check param bounds.
		if(nodeVideoIndex >= nVideos)
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch video bounds error\n"));
			return FALSE;
		}

		// Add all videos to the tree.
		CString str;
		HTREEITEM hVideoItem = NULL;
		HTREEITEM hSelVideoItem = NULL;
		for(int iVideo = 0; iVideo < nVideos; iVideo++)
		{
			CVideo & video = parentTopic.videos[iVideo];

			// Get the Title.
			str = parentTopic.videos[iVideo].title;

			// Add the postfix marker
			str += VID_POSTFIX;

			// Insert hItem video.
			hVideoItem = m_treeCtrl.InsertItem(str, hParentItem, TVI_LAST);

			// Ensure.
			if(!hVideoItem)
			{
				TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch hVideoItem NULL\n"));
				return FALSE;
			}

			// If at the video searched for.
			if(iVideo == nodeVideoIndex)
			{
				// Set pDoc members.
				pDoc->m_curVideo = video;
				pDoc->m_hCurVideo = hVideoItem;
				
				// Set string array for view display.
				pDoc->SetDisplayStringsVideo();

				// Save handle.
				hSelVideoItem = hVideoItem;
			}
		}
		// Set the selection.
		if(hSelVideoItem)
		{
			m_treeCtrl.SelectItem(hSelVideoItem);
			return TRUE;
		}
		// Error
		else
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch hSelVideoItem NULL\n"));
			return FALSE;
		}
	}

	/////////////////////////////////
	// Else traverse to child topic.
	else if(iNodeArParent < nodeArSz - 1 && nChildTopics)
	{
		CString topicTitleStr, treeTopicStr;
		HTREEITEM hNewItem;
		int iNextNode = 0;

		// Add all child topics to tree.
		for(int iTopic = 0; iTopic < nChildTopics; iTopic++)
		{
			// Get child Topic ref.
			CTopic & childTopic = parentTopic.childTopics[iTopic];

			// Get topicTree title str.
			topicTitleStr = childTopic.title;

			// Get topicTree topic str.
			treeTopicStr = childTopic.treeTopic;

			// Insert child below parent.
			TRACE(_T("Inserting child %d : %s\n"), iTopic, topicTitleStr);
			hNewItem = m_treeCtrl.InsertItem(topicTitleStr, hParentItem, TVI_LAST);
			// MFC calls:
			// Note TVINSERTSTRUCT tvis - item.mask == 1 (TVIF_TEXT)
			// SendMessage(m_hWnd, TVM_INSERTITEM, 0, (LPARAM)&tvis)

			// Ensure.
			if(!hNewItem)
			{
				TRACE(_T("ERROR CTreeDialog ExpandChildTopicSearch hNewItem NULL\n"));
				return FALSE;
			}

			// Fix 9/13/2017 - duplicate topic title fix
			// Save element this item is stored under.
			// Element is only valid for this tree indent. 
			m_treeCtrl.SetItemData(hNewItem, (DWORD) iTopic);

			// If at the next child treeTopic on the node path,
			// make a recursive call.
			iNextNode = iNodeArParent + 1;
			if(iNextNode < nodeArSz				 && 
			   treeTopicStr == nodeAr[iNextNode] )
			{
				TRACE(_T("CTreeDialog ExpandChildTopicSearch - recursive call\n"));

				// Expand this child topic.
				if(!ExpandChildTopicSearch(childTopic, hNewItem, nodeAr, 
										   nodeVideoIndex, iNextNode))
				{
					return FALSE;
				}
				TRACE(_T("CTreeDialog ExpandChildTopicSearch - post recursive call\n"));
			}

			// Else add dummy item below child.
			else
			{
				// If Topic has videos or child topics, add a dummy child.
				if(childTopic.childTopics.GetSize() || 
				   childTopic.videos.GetSize()		)
				{
					TRACE(_T("Inserting and collapsing DUMMY_ITEM\n"));
					m_treeCtrl.InsertItem(DUMMY_ITEM, hNewItem, hNewItem);
					m_treeCtrl.Expand(hNewItem, TVE_COLLAPSE);	// collapse the dummy, show the +
				}

				// Some Topics are empty.
				else
				{
					TRACE(_T("Inserting and collapsing EMPTY_TOPIC\n"));
					m_treeCtrl.InsertItem(EMPTY_TOPIC, hNewItem, hNewItem);
					m_treeCtrl.Expand(hNewItem, TVE_COLLAPSE);	// collapse the dummy, show the +
				}
			}
		}
	}

	// dbg
	else
		return FALSE;

///////////////////////////////

/*	CString topicTitleStr, treeTopicStr;
	int nTopics = parentTopic.childTopics.GetSize();
	int nVideos = parentTopic.videos.GetSize();
	int iTopic, iVideo;
	HTREEITEM hNewItem;

	// Add all child topics to tree.
	for(iTopic = 0; iTopic < nTopics; iTopic++)
	{
		// Get child Topic ref.
		CTopic & childTopic =  parentTopic.childTopics[iTopic];

		// Get topicTree title str.
		topicTitleStr = childTopic.title;

		// Get topicTree topic str.
		treeTopicStr = childTopic.treeTopic;

		// Insert child below parent.
		TRACE(_T("Inserting child %d : %s\n"), iTopic, topicTitleStr);
		hNewItem = m_treeCtrl.InsertItem(topicTitleStr, hParentItem, TVI_LAST);
		// MFC calls:
		// Note TVINSERTSTRUCT tvis - item.mask == 1 (TVIF_TEXT)
		// SendMessage(m_hWnd, TVM_INSERTITEM, 0, (LPARAM)&tvis)

		// Ensure.
		if(!hNewItem)
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopic hNewItem NULL\n"));
			return FALSE;
		}

		// Fix 9/13/2017 - duplicate topic title fix
		// Save element this item is stored under.
		// Element is only valid for this tree indent. 
		m_treeCtrl.SetItemData(hNewItem, (DWORD) iTopic);

		// If at the next child treeTopic on the path,
		// make a recursive call.
		if(iNodeArParent + 1 <= nodeArSz			 &&
		   treeTopicStr == nodeAr[iNodeArParent + 1] )
		{
			TRACE(_T("CTreeDialog ExpandChildTopic - recursive call\n"));

			// Expand this child topic.
	//		if(!ExpandChildTopic(childTopic, curVideo, nodeAr, 
	//							 hNewItem, iNodeArParent + 1))
	//		{
	//			return FALSE;
	//		}
			TRACE(_T("CTreeDialog ExpandChildTopic - post recursive call\n"));
		}

		// Else add dummy item below child.
		else
		{
			// If Topic has videos or child topics, add a dummy child.
			if(childTopic.childTopics.GetSize() || 
			   childTopic.videos.GetSize()		)
			{
				TRACE(_T("Inserting and collapsing DUMMY_ITEM\n"));
				m_treeCtrl.InsertItem(DUMMY_ITEM, hNewItem, hNewItem);
				m_treeCtrl.Expand(hNewItem, TVE_COLLAPSE);	// collapse the dummy, show the +
			}

			// Some Topics are empty.
			else
			{
				TRACE(_T("Inserting and collapsing EMPTY_TOPIC\n"));
				m_treeCtrl.InsertItem(EMPTY_TOPIC, hNewItem, hNewItem);
				m_treeCtrl.Expand(hNewItem, TVE_COLLAPSE);	// collapse the dummy, show the +
			}
		}
	}

	// Add all videos to the tree.
	// If at the target video, set as current selection.
	// Note a Topic may have vids and sub-topics.
	CString str;
	HTREEITEM hVideoItem = NULL;
	HTREEITEM hSelVideoItem = NULL;
	for(iVideo = 0; iVideo < nVideos; iVideo++)
	{
		CVideo & video = parentTopic.videos[iVideo];

		// Get the Title.
		str = parentTopic.videos[iVideo].title;

		// Add the postfix marker
		str += VID_POSTFIX;

		// Insert hItem video.
		hVideoItem = m_treeCtrl.InsertItem(str, hParentItem, TVI_LAST);

		// Ensure.
		if(!hVideoItem)
		{
			TRACE(_T("ERROR CTreeDialog ExpandChildTopic hVideoItem NULL\n"));
			return FALSE;
		}

		// If the video we are after.
		if(curVideo.title == video.title)
		{
			// Set pDoc members.
			pDoc->m_curVideo = video;
			pDoc->m_hCurVideo = hVideoItem;
			
			// Set string array for view display.
			pDoc->SetDisplayStringsVideo();

			// Save handle.
			hSelVideoItem = hVideoItem;
		}
	}
	// Done
	//
	// Highlite treeCtrl Video item if inserted.
	if(hSelVideoItem)
		m_treeCtrl.SelectItem(hSelVideoItem);
*/
	// Ok
	return TRUE;
}

void CTreeDialog::OnDestroy() 
{
	CDialog::OnDestroy();
	
	// TODO: Add your message handler code here

	// Clean CApp handle.
	((CApp *)AfxGetApp())->m_hwndTreeDlg = NULL;	
}

// Tree item expanding.
//
// Note any videos displayed will have their 
// prefixes drawn in OnNotify().
void CTreeDialog::OnItemexpandingKhanTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TRACE(_T("Entering CTreeDialog OnItemexpandingKhanTree\n"));

	// Result = 0 for control to expand or collapse.
	*pResult = 0;

	// OnSetSelLastViewed() will generate a TVN_ITEMEXPANDING message
	// when a new node is inserted. Exactly why is unknown.
	// We must bail here or all sorts of duplicate items are added.
	// Note flag is set and cleared in OnSetSelLastViewed().
	if(m_manualExpand)
	{
		NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
		BOOL expand = pNMTreeView->action == TVE_EXPAND;
		TRACE(_T("CTreeDialog OnItemexpandingKhanTree expanding = %d Skipping\n"), expand);
		return;
	}

	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	HTREEITEM hItem = pNMTreeView->itemNew.hItem;

	CTreeCtrl * pTreeCtrl = (CTreeCtrl*)GetDlgItem(IDC_KHAN_TREE);

	// Fix 9/13/2017 - duplicate topic title fix
	// Get this items parent topic tree nodes and their ar indices.
	CNodeArray nodeAr;
	if(!GetPathNodesFromItem(hItem, nodeAr))
		return;	

	// Expanding
	if(pNMTreeView->action == TVE_EXPAND)
	{
		TRACE(_T("In CTreeDialog OnItemexpandingKhanTree TVE_EXPAND\n"));

		// Get CDoc ptr.
		CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;

		if(pDoc)
		{
			CTopicTree & tree = pDoc->GetTree();
			CTopic topic;

			// Delete any dummy item.
			if(pTreeCtrl->ItemHasChildren(hItem))
			{
			   HTREEITEM hChild = pTreeCtrl->GetChildItem(hItem);
			   if(hChild && pTreeCtrl->GetItemText(hChild) == DUMMY_ITEM)
				   pTreeCtrl->DeleteItem(hChild);
			}

			// Fix 9/13/2017 - duplicate topic title fix
			// Get the expanding topic.
			if(tree.GetTopicByPos(topic, nodeAr))
			{
				HTREEITEM newItem;
				CString str;
				int i;
				int nVideos = topic.videos.GetSize();
				int nTopics = topic.childTopics.GetSize();

				// If topic has videos.
				for(i = 0; i < nVideos; i++)
				{
					CVideo & vid = topic.videos[i];

					// And the Title.
					str = topic.videos[i].title;

					// Add the postfix marker
					str += VID_POSTFIX;

					// Insert hItem videos.
					pTreeCtrl->InsertItem(str, hItem, TVI_LAST);
				}

				// If topic has child Topics.
				for(i = 0; i < nTopics; i++)
				{
					// dbg
					str = topic.childTopics[i].title;

					// Insert the hItem parent topic.
					newItem = pTreeCtrl->InsertItem(str, hItem, TVI_LAST);

					// Fix 9/13/2017 - duplicate topic title fix
					// Save element this item is stored under.
					// Element is only valid for this tree indent. 
					pTreeCtrl->SetItemData(newItem, (DWORD) i);

					// If hItem Topic has videos or child topics, add a dummy child.
					if(topic.childTopics[i].childTopics.GetSize() ||
					   topic.childTopics[i].videos.GetSize())
					{
						pTreeCtrl->InsertItem(DUMMY_ITEM, newItem, newItem);
					}

					// Some Topics are empty. Should not happen.
					else
					{
						pTreeCtrl->InsertItem(EMPTY_TOPIC, newItem, newItem);
						TRACE(_T("WARNING CTreeDialog OnItemexpandingKhanTree - empty topic\n"));
					}
				}
				// Highlite
				pTreeCtrl->SelectItem(hItem);

				// Error?
				if(!nVideos && !nTopics)
				{
				//	*pResult = 1;
				//	return;
				}
			}
		}
	}

	// Else pNMTreeView->action == TVE_COLLAPSE
	else
	{
		TRACE(_T("In CTreeDialog OnItemexpandingKhanTree TVE_COLLAPSE\n"));

		// dbg - one call per item no matter expanded children
		//CString s = pTreeCtrl->GetItemText(hItem);

		// Highlite
		pTreeCtrl->SelectItem(hItem);

		// If an EMPTY_TOPIC.
		if(pTreeCtrl->ItemHasChildren(hItem))
		{
		   HTREEITEM hChild = pTreeCtrl->GetChildItem(hItem);
		   if(hChild && pTreeCtrl->GetItemText(hChild) == EMPTY_TOPIC)
		   {
				// Highlite parent and bail.
			//	pTreeCtrl->SelectItem(hItem);
				return;
		   }
		}

		// Otherwise, since we are collapsing, we know there 
		// are child topics or videos.
		//
		// Remove them all from the control.
		DeleteAllChildren(hItem);

		// Add a dummy.
		pTreeCtrl->InsertItem(DUMMY_ITEM, hItem);
	}
}

// Get all treeCtrl parents of hItem into nodes array.
BOOL CTreeDialog::GetPathFromItemOld(HTREEITEM hItem, CStringArray & nodes)
{
	if(!hItem)
		return FALSE;

	CTreeCtrl * pTreeCtrl = (CTreeCtrl*)GetDlgItem(IDC_KHAN_TREE);

	if(!pTreeCtrl)
		return FALSE;

	// Add param item str.
    CString str = pTreeCtrl->GetItemText(hItem);
	if(str.IsEmpty())
		return FALSE;

	int elmt = nodes.Add(str);

    HTREEITEM hParent = hItem;
    while ((hParent = pTreeCtrl->GetParentItem(hParent)) != NULL) 
	{
		// Get Parent str.
        str = pTreeCtrl->GetItemText(hParent);
		if(str.IsEmpty())
			return FALSE;

		// Insert at front of list.
		nodes.InsertAt(0, str);

		// Fix 9/13/2017 - duplicate topic title fix
		// Get the index saved for this parent item.
		// Element is only valid for this tree indent. 
		// CUIntArray
		UINT index = pTreeCtrl->GetItemData(hParent);

    }

	if(nodes.GetSize() == 0)
		return FALSE;

    return TRUE;
}

// Fix 9/13/2017 - duplicate topic title fix
// Get all treeCtrl parent nodes of hItem into nodes array.
BOOL CTreeDialog::GetPathNodesFromItem(HTREEITEM hItem, CNodeArray & nodeAr)
{
	if(!hItem)
		return FALSE;

	CTreeCtrl * pTreeCtrl = (CTreeCtrl*)GetDlgItem(IDC_KHAN_TREE);

	if(!pTreeCtrl)
		return FALSE;

	CNodeItem nodeItem;

	// Get item str.
    nodeItem.m_title = pTreeCtrl->GetItemText(hItem);
	if(nodeItem.m_title.IsEmpty())
		return FALSE;

	// Get item index
	nodeItem.m_arIndex = pTreeCtrl->GetItemData(hItem);

	// Add to node list.
	int elmt = nodeAr.Add(nodeItem);

	// Get parent nodes of hItem.
    HTREEITEM hParent = hItem;
    while ((hParent = pTreeCtrl->GetParentItem(hParent)) != NULL) 
	{
		nodeItem.Zero();

		// Get parent item str.
		nodeItem.m_title = pTreeCtrl->GetItemText(hParent);
		if(nodeItem.m_title.IsEmpty())
			return FALSE;

		// Get item index
		nodeItem.m_arIndex = pTreeCtrl->GetItemData(hParent);

		// Insert at front of list.
		nodeAr.InsertAt(0, nodeItem);
    }

	if(nodeAr.GetSize() == 0)
		return FALSE;

    return TRUE;
}

// Delete all child items from hItem param.
// Note Expand(hRoot, TVE_COLLAPSERESET) apparently does not
// clean up the hItem's sub-tree items.
// Also, dummy items must be added by caller
// to display a + sign.
void CTreeDialog::DeleteAllChildren(HTREEITEM hItem)
{
	CTreeCtrl * pTreeCtrl = (CTreeCtrl*)GetDlgItem(IDC_KHAN_TREE);
	HTREEITEM hChildItem;

	// Ensure.
	if((hChildItem = pTreeCtrl->GetChildItem(hItem)) == NULL)
		return;

	// Walk all children and children of.
	do 
	{
		HTREEITEM hNextItem = pTreeCtrl->GetNextSiblingItem(hChildItem);
		pTreeCtrl->DeleteItem(hChildItem);
		hChildItem = hNextItem;
	} 
	while(hChildItem != NULL);
}

// User changing treeCtrl items.
void CTreeDialog::OnSelchanging(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here

	*pResult = 0;

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;

	// If a video is running, it's best to keep the
	// selection on it so the user knows what is 
	// being watched.
	if(!pDoc || pDoc->m_videoPlaying)
		*pResult = 1;	
}


// Get selected Video data.
void CTreeDialog::OnSelchangedKhanTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here
	
	*pResult = 0;

	HTREEITEM hSelItem = pNMTreeView->itemNew.hItem;

	if(!SetCurrentSelection(hSelItem))
		*pResult = 1;
}

// Set CDoc data to the current treeCtrl selection.
BOOL CTreeDialog::SetCurrentSelection(HTREEITEM hSelItem) 
{
	CTreeCtrl * pTreeCtrl = (CTreeCtrl*)GetDlgItem(IDC_KHAN_TREE);
	CString selStr = pTreeCtrl->GetItemText(hSelItem);

	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return FALSE;

	////////////////////////////////
	// If a video has been selected.
	CString s = selStr.Right(VID_POSTFIX_LEN);
	if(s == VID_POSTFIX)
	{
		// Clean postfix.
		selStr.TrimRight();	// VID_POSTFIX

		CTopicTree & tree = pDoc->GetTree();
		CTopic topic;
		CVideo video;

		// Get parent item.
		HTREEITEM hParent = pTreeCtrl->GetParentItem(hSelItem);
		CString strParent = pTreeCtrl->GetItemText(hParent);

		// Fix 9/13/2017 - duplicate topic title fix
		// Get this items parent topic tree nodes and their ar indices.
		CNodeArray nodeAr;
		if(!GetPathNodesFromItem(hParent, nodeAr))
			return FALSE;

		// Fix 9/13/2017 - duplicate topic title fix
		// Get the video's Parent topic.
		if(tree.GetTopicByPos(topic, nodeAr))
		{
			int nVideos = topic.videos.GetSize();

			// Get the video.
			for(int i = 0; i < nVideos; i++)
			{
				if(topic.videos[i].title == selStr)
				{
					pDoc->m_curVideo = topic.videos[i];
					pDoc->m_hCurVideo = hSelItem;
					
					// Set string array for view display.
					pDoc->SetDisplayStringsVideo();
					break;
				}
			}
		}
		// Clean topic.
		pDoc->m_curTopic.Zero();
		pDoc->m_hCurTopic = NULL;
	}

	///////////////////////
	// Else get the topic.
	else
	{
		CTopicTree & tree = pDoc->GetTree();
		CTopic topic;

		// Fix 9/13/2017 - duplicate topic title fix
		// Get this items parent topic tree nodes and their ar indices.
		CNodeArray nodeAr;
		if(!GetPathNodesFromItem(hSelItem, nodeAr))
			return FALSE;

		// Fix 9/13/2017 - duplicate topic title fix
		// Get the video's Parent topic.
		if(tree.GetTopicByPos(topic, nodeAr))
		{
			int nChildTopics = topic.childTopics.GetSize();
			int nVideos = topic.videos.GetSize();

			// Set Cur Topic data.
			pDoc->m_curTopic = topic;
			pDoc->m_hCurTopic = hSelItem;
			pDoc->m_nChildTopics = nChildTopics;
			pDoc->m_nVideos = nVideos;
			pDoc->m_nTotalVideoTime = 0;
			for(int i = 0; i < nVideos; i++)
			{
				pDoc->m_nTotalVideoTime += _wtoi(topic.videos[i].duration);
			}

			// Set CDoc string array for view display.
			pDoc->SetDisplayStringsTopic();
		}
		// Clean cur video.
		pDoc->m_curVideo.Zero();
		pDoc->m_hCurVideo = NULL;
	}
	return TRUE;
}

// Set Info Tip string for selected Topic or Video.
//
// Show a video information info tip for a running video.
// Lets user review info while watching.
//
// 4/15/15 
//	- added buf length checks - was crashing with some vids.
//	- removed code for topics as it's not needed and does
//	  not get called.
void CTreeDialog::OnTvnGetInfoTip(NMHDR* pNMHDR, LRESULT* pResult) //TVN_GETINFOTIPW (Unicode) 
{
	*pResult = 0;

	LPNMTVGETINFOTIP pGetInfoTip = (LPNMTVGETINFOTIP)pNMHDR;

	// Ensure
	if(pGetInfoTip == NULL)
		return;

	// Use info tips only when a video is playing, 
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc || !pDoc->m_videoPlaying)
	{
		return;
	}

	// Ensure
	if(pGetInfoTip->hItem == NULL)
		return;

	// Get max chars. Limit to 250, else tip is too big.
	int maxChars = pGetInfoTip->cchTextMax - 10; //1024
	if(maxChars > 250)
		maxChars = 250;

	// If InfoTip is for the currently selected Video.
	if(pGetInfoTip->hItem == pDoc->m_hCurVideo)
	{
		int time = _wtoi(pDoc->m_curVideo.duration);

		// Ensure.
		if(!time)
			return;

		int minutes = time / 60;
		int seconds = time % 60;
		CString str;
		str.Format(_T("Video Title:  %s\nDescription:  %s\nDuration:  %d Min. %d Sec."), 
								pDoc->m_curVideo.title,
								pDoc->m_curVideo.description,
								minutes, seconds);

		// Truncate string if needed - Will crash app if over cchTextMax chars.
		if(str.GetLength() > maxChars)
		{
			// Truncate only description if possible.
			CString fmt = _T("Video Title:  \nDescription:  \nDuration:   Min.  Sec.");
			int frmtChars = fmt.GetLength() + 3;
			int maxDesc = maxChars - (pDoc->m_curVideo.title.GetLength() + frmtChars);
			if(maxDesc)
			{
				CString desc = pDoc->m_curVideo.description;				
				int n = desc.Replace(_T("\\n"), _T(" "));	// Clean any newlines.
				desc.Delete(maxDesc, desc.GetLength() - maxDesc);
				desc += _T("...");
				str.Format(_T("Video Title:  %s\nDescription:  %s\nDuration:  %d Min. %d Sec."), 
										pDoc->m_curVideo.title,
										desc,
										minutes, seconds);

			}
			// Or just truncate what is there.
			else
			{
				str.Replace(_T("\\n"), _T(" "));					// Clean any newlines.
				str.Delete(maxChars, str.GetLength() - maxChars);
			}
		}
		// Else clean up.
		else
		{
			str.Replace(_T("\\n"), _T(" "));					// Clean any newlines.
		}

		// Copy to buffer.
		//wcscpy(pGetInfoTip->pszText, str);					// VC6 KhanVlc
		StringCbCopy(pGetInfoTip->pszText, 
					 (str.GetLength() + 1) * sizeof(TCHAR),		// VC6 KhanDS
					 str);
	}

	// Else a tooltip.
	else
	{
		int g = 0;
	}
}

// Context Menu handler.
void CTreeDialog::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	// Get item right clicked
	UINT uFlags;
	HTREEITEM hItem;	
	hItem = m_treeCtrl.HitTest(point, &uFlags);
	if(hItem == NULL)
		return;

	// Get Doc and tree.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;
	CTopicTree & tree = pDoc->GetTree();

	// If ListFiles is showing, bail.
	if(pDoc->m_listFiles.ListShowing())
		return;

	// Flags
	BOOL downloading = pDoc->DownloadingVideos();
	BOOL vidPlaying = pDoc->m_videoPlaying;

	// Array of videos for download.
	// Size is set in GetTopicVideos().
	CVideoArray downloadAr;
	int vidsOnList = 0;

	// Get item text.
	CString s = m_treeCtrl.GetItemText(hItem);

	// Create the context.
	CMenu menu;
	menu.CreatePopupMenu();
	m_treeCtrl.ClientToScreen(&point);
	CString str;

	// If not the current selection, select.
	if(hItem != pDoc->m_hCurTopic && hItem != pDoc->m_hCurVideo)
	{
		// Use CTreeCtrl to select.
		m_treeCtrl.SelectItem(hItem);
	}

	//
	// Selection should now be set. Add context menu items.
	//
	
	// If Topic selected.
	if(hItem == pDoc->m_hCurTopic)
	{
		// Get all videos for the Topic and its children. 
		int nVids = tree.GetTopicVideos(pDoc->m_curTopic, &downloadAr);
		if(nVids)
		{
			// Sort vids alphabetically, and remove those listed 
			// in multiple topics - for example Math has 411 dups
			// of 2110 total.
			downloadAr.SortVideoTitlesAscendRemoveDuplicates();

			// Remove vids currently on disk, and count 
			// vids currently on the download list.
			nVids = downloadAr.GetSize();
			for(int i = nVids - 1; i >= 0; i--)
			{
				// Remove vids already on disk.
				if(pDoc->VideoOnDisk(downloadAr[i]))
				{
					downloadAr.RemoveAt(i);
					continue;
				}
				// Count vids already on the download list.
				if(pDoc->VideoForDownload(downloadAr[i]))
					vidsOnList ++;
			}

			// Num vids not on disk.
			nVids = downloadAr.GetSize();

			// Num vids we can add to list.
			int toAdd = nVids - vidsOnList;

			// If any available for download, allow adding to list.
			if(toAdd > 0)
			{
				str.Format(_T("Add %d videos to download list"), toAdd);
				menu.AppendMenu(MF_STRING, ID_CTX_DOWNLOAD_ALL, str);
			}

			// Else if any marked for download, allow removing from 
			// list if not currently downloading videos.
			else if(vidsOnList && !downloading)
			{
				str.Format(_T("Remove %d videos from download list"), vidsOnList);
				menu.AppendMenu(MF_STRING, ID_CTX_DOWNLOAD_NONE, str);
			}
		}
	}

	// If a Video is selected.
	else if(hItem == pDoc->m_hCurVideo)
	{
		// Play from disk.
		if(pDoc->VideoOnDisk(pDoc->m_curVideo)		 &&
		   !pDoc->VideoForDownload(pDoc->m_curVideo) )	// not on disk and being written.
		{
			if(!vidPlaying)
				menu.AppendMenu(MF_STRING, ID_CTX_PLAY_DISK, _T("Play from disk"));

			// Add delete History.
			if(!vidPlaying)
			{
				if(!pDoc->m_history.BeenViewed(pDoc->m_curVideo))
					menu.AppendMenu(MF_STRING, ID_CTX_ADD_HISTORY, _T("Mark as Viewed"));
				else
					menu.AppendMenu(MF_STRING, ID_CTX_DELETE_HISTORY, _T("Remove from history"));
			}
		}

		// Play stream, or download to disk.
		else
		{
			// Play.
			if(!vidPlaying)
				menu.AppendMenu(MF_STRING, ID_CTX_PLAY_STREAM, _T("Play from Khan server"));

			// Download.
			if(!pDoc->VideoForDownload(pDoc->m_curVideo))
				menu.AppendMenu(MF_STRING, ID_CTX_DOWNLOAD_VID, _T("Add video to download list"));

			// Remove video marked for download.
			else if(pDoc->VideoForDownload(pDoc->m_curVideo) &&
					!downloading )
				menu.AppendMenu(MF_STRING, ID_CTX_DOWNLOAD_VID_NO, _T("Remove video from ")
																   _T("download list")    );
			// Add to History.
			if(!vidPlaying)
			{
				if(!pDoc->m_history.BeenViewed(pDoc->m_curVideo))
					menu.AppendMenu(MF_STRING, ID_CTX_ADD_HISTORY, _T("Mark as Viewed"));
				else
					menu.AppendMenu(MF_STRING, ID_CTX_DELETE_HISTORY, _T("Remove from history"));
			}
		}
	}
	// Error - selection.
	else
	{
		TRACE(_T("CTreeDialog OnContextMenu no sel item\n"));
		return;
	}

	// If context menu items added.
	if(menu.GetMenuItemCount() > 0)
	{
		// Show Context Menu.
		int nCmd = menu.TrackPopupMenu(	TPM_LEFTALIGN  | 
										TPM_LEFTBUTTON |
										TPM_RETURNCMD,		// see sdk TrackPopupMenu
										point.x, point.y, 
										this);

		//////////////////////////
		// Handle user selection.
		BOOL updateList = FALSE;
		BOOL redraw = FALSE;
		CRect rect, invalid;
		invalid.SetRectEmpty();
		
		// Mark a video for download.
		if(nCmd  == ID_CTX_DOWNLOAD_VID && 
		   hItem == pDoc->m_hCurVideo   )
		{
			// Add to list.
			pDoc->AddVideoForDownload(pDoc->m_curVideo);

			// Set clip for redraw later.
			m_treeCtrl.GetItemRect(hItem, invalid, FALSE);
			redraw = TRUE;
			
			updateList = TRUE;
		}

		// Un-mark a video for download.
		else if(nCmd == ID_CTX_DOWNLOAD_VID_NO && 
				hItem == pDoc->m_hCurVideo  )
		{
			// If marked for download. Note marked vids are not on disk.
			if(pDoc->VideoForDownload(pDoc->m_curVideo))
			{
				m_treeCtrl.GetItemRect(hItem, invalid, FALSE);
				redraw = TRUE;

				pDoc->RemoveVideoForDownload(pDoc->m_curVideo);
				updateList = TRUE;
			}
		}

		// Download all videos in the current topic
		// and its children.
		else if(nCmd  == ID_CTX_DOWNLOAD_ALL &&
				hItem == pDoc->m_hCurTopic   )
		{
			// Add vids not currently on the download list.
			// Note AddVideoForDownload() checks for duplicates.
			int sz = downloadAr.GetSize();
			for(int i = 0; i < sz; i++)
			{
				// Add to list. 
				pDoc->AddVideoForDownload(downloadAr[i]);
			}
			// Update any video marks and the 
			// download list.
			redraw = TRUE;
			updateList = TRUE;
		}

		// Remove all Topic vids added to the download list.
		else if(nCmd == ID_CTX_DOWNLOAD_NONE &&
				hItem == pDoc->m_hCurTopic   )
		{
			// Remove from the download list.
			int sz = downloadAr.GetSize();
			for(int i = 0; i < sz; i++)
			{
				// If on list, remove. 
				if(pDoc->VideoForDownload(downloadAr[i]))
				{
					pDoc->RemoveVideoForDownload(downloadAr[i]);
				}
			}
			// Update any video marks and the 
			// download list.
			redraw = TRUE;
			updateList = TRUE;
		}

		// Play video from disk.
		else if(nCmd == ID_CTX_PLAY_DISK)
		{
			HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
			if(hwndView)
				::PostMessage(hwndView, WM_USER_VID_START, 0, 0);

		//	pDoc->PlayCurrentVideo(); VC6 KhanVLC
		}

		// Play video from khan server.
		else if(nCmd == ID_CTX_PLAY_STREAM)
		{
			HWND hwndView = ((CApp*)AfxGetApp())->m_hwndView;
			if(hwndView)
				::PostMessage(hwndView, WM_USER_VID_START, 0, 0);
		//	pDoc->PlayCurrentVideo();
		}

		// Add video to History.
		else if(nCmd == ID_CTX_ADD_HISTORY && 
				hItem == pDoc->m_hCurVideo )
		{
			BOOL onDisk = pDoc->VideoOnDisk(pDoc->m_curVideo);

			pDoc->m_history.AddHistoryItemCheater(pDoc->m_curVideo);

			m_treeCtrl.GetItemRect(hItem, invalid, FALSE);
			redraw = TRUE;

			// Update View Strings.
			pDoc->SetDisplayStringsVideo();
		}

		// Delete video History.		
		else if(nCmd == ID_CTX_DELETE_HISTORY && 
				hItem == pDoc->m_hCurVideo )
		{
			pDoc->m_history.DeleteHistory(pDoc->m_curVideo);

			m_treeCtrl.GetItemRect(hItem, invalid, FALSE);
			redraw = TRUE;

			// Update View Strings.
			pDoc->SetDisplayStringsVideo();
		}

		// Redraw all videos currently visable so as
		// to update any changed video marks.
		if(redraw)
		{
			CString str, strPost;
			invalid.SetRectEmpty();
			HTREEITEM hItem = m_treeCtrl.GetFirstVisibleItem();
			while(hItem)
			{
				str = m_treeCtrl.GetItemText(hItem);
				strPost = str.Right(VID_POSTFIX_LEN);
				if(strPost == VID_POSTFIX)
				{
					// Add to clip.
					m_treeCtrl.GetItemRect(hItem, rect, FALSE);
					invalid.UnionRect(invalid, rect);
					redraw = TRUE;

				}
				// Get next visable.
				hItem = m_treeCtrl.GetNextVisibleItem(hItem);
			}
			m_treeCtrl.InvalidateRect(&invalid);
			m_treeCtrl.UpdateWindow();
		}

		// Update the DownloadDlg's list.
		if(updateList)
		{
			HWND hwnd = ((CApp *)AfxGetApp())->m_hwndDownloadDlg;
			// Update the download dialog list.
			if(hwnd)
				::PostMessage(hwnd, WM_USER_DLG_UPDATE, 0, 0);	
		}
	}
}

// Change the Tree Ctrl font.
// Called via message from CTabView::OnSetupTreeFont()
// for setup menu item.
void CTreeDialog::OnSetupTreeFont() 
{
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	if(pDoc)
	{
		CFontSetup dlg;
		dlg.SetCurFont(pDoc->m_treeLf);
		if(dlg.DoModal() == IDOK)
		{
			LOGFONT lf = dlg.GetLogfont();

			// Set up font.
			if((HFONT)m_treeFont)
				m_treeFont.DeleteObject();
			pDoc->m_treeLf = lf;
			m_treeFont.CreateFontIndirect(&lf);
			m_treeCtrl.SetFont(&m_treeFont, TRUE);
		}
	}	
}

// Ignore Return, Escape and Space keys.
BOOL CTreeDialog::PreTranslateMessage(MSG* pMsg) 
{
	// TODO: Add your specialized code here and/or call the base class

	if(pMsg->message == WM_KEYDOWN		&&
	   ( pMsg->wParam == VK_RETURN		|| 
		 pMsg->wParam == VK_ESCAPE		||
		 pMsg->wParam == VK_SPACE )		)
	{
		return TRUE;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}

// Draw Video Prefixes.
BOOL CTreeDialog::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult) 
{
	LPNMHDR pNmhdr = (LPNMHDR)lParam;

	switch(pNmhdr->code)
	{
		case NM_CUSTOMDRAW:
		{
			LPNMTVCUSTOMDRAW pCustomDraw = (LPNMTVCUSTOMDRAW)lParam;
			switch(pCustomDraw->nmcd.dwDrawStage)
			{
				case CDDS_PREPAINT:
				{
					// Set pResult to CDRF_NOTIFYITEMDRAW to get 
					// the CDDS_ITEMPREPAINT notification below. 
					*pResult = CDRF_NOTIFYITEMDRAW;
					return true;
				}

				case CDDS_ITEMPREPAINT:
				{
					// If Root expanded.
					if(pCustomDraw->iLevel)
					{
						HTREEITEM hItem = (HTREEITEM)(pCustomDraw->nmcd.dwItemSpec);
						int g = 0; // dwitemspec
						if(hItem)
						{
							CString s = m_treeCtrl.GetItemText(hItem);
							CString post = s.Right(VID_POSTFIX_LEN);
							if(post == VID_POSTFIX)
							{
								CDC * pDC = CDC::FromHandle(pCustomDraw->nmcd.hdc);
								DrawVideoMarks(hItem, pDC);
							}
						}
					}					
					*pResult = CDRF_SKIPDEFAULT;
					return false;
				}
			}
		}
		break;
	}
	// Base class
	return CDialog::OnNotify(wParam, lParam, pResult);
}

// Draw prefixes for a Video.
// Called from OnNotify() only.
void CTreeDialog::DrawVideoMarks(HTREEITEM hItem, CDC * pDC)
{
	// Need a CDoc ptr.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// Get all parents of this video item.
	HTREEITEM hParent = m_treeCtrl.GetParentItem(hItem);

	// Fix 9/13/2017 - duplicate topic title fix
	// Get this items parent topic tree nodes and their ar indices.
	CNodeArray nodeAr;
	if(!hParent || !GetPathNodesFromItem(hParent, nodeAr))
		return;

	// Get the Parent Topic.
	CTopicTree & tree = pDoc->GetTree();
	CTopic topic;

	// Fix 9/13/2017 - duplicate topic title fix
	// Get the Parent topic.
	if(tree.GetTopicByPos(topic, nodeAr))
	{
		// Get item text - the video title.
		CString str = m_treeCtrl.GetItemText(hItem);
		// Remove postfix.
		str.TrimRight();

		int sz = topic.videos.GetSize();

		// Linear search the topic for the video.
		for(int i = 0; i < sz; i++)
		{
			if(topic.videos[i].title == str)
			{
				// Get a ref.
				CVideo & vid = topic.videos[i];

				CRect rect, bound;
				CBrush brush;
				brush.CreateSolidBrush(RGB(0, 0, 0));

				// Build rect bounding prefixes.
				//
				m_treeCtrl.GetItemRect(hItem, &bound, TRUE);
				 
				// Adjust as a percentage so we don't close up.
				double vert = bound.Height();
				int shift = (int)floor(vert * .125);
				
				// Adjust prefix canvas.
				// Note left = 52 @ 1st, 78 @ 2nd, 104 @ 3rd tree step - all font sizes.
				bound.right	  = bound.left - shift * 2; 
				bound.left	  = bound.right - 12;
				bound.top	 += shift;
				bound.bottom -= shift;

				CPoint cent = bound.CenterPoint();

				// If marked for download.
				if(pDoc->VideoForDownload(vid))
				{
					rect = bound;
					rect.top = cent.y - 1;
					rect.bottom = cent.y + 1;
					pDC->FillRect(&rect, &brush);
				}
				else
				{
					// If on disk.
					if(pDoc->VideoOnDisk(vid))
					{
						rect = bound;
						rect.top = cent.y - 2;
						rect.bottom = cent.y + 3;
						rect.right = rect.left + 5;
						pDC->FillRect(&rect, &brush);
					}

					// If viewed.
					if(pDoc->m_history.BeenViewed(vid))
					{
						rect = bound;
						rect.left = rect.right - 2;
						pDC->FillRect(&rect, &brush);
					}

					// Else the item is for streaming.
					else
					{
						// no marks.
					}
				}
				// Done
				return;
			}
		}
	}
	// Not a video item.
	return;
}








